# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

from typing import Any, Dict, List, Literal, Mapping, Optional, TYPE_CHECKING, Union, overload

from .. import _model_base
from .._model_base import rest_discriminator, rest_field
from ._enums import (
    CommunicationMessageKind,
    CommunicationMessagesChannel,
    MessageActionBindingKind,
    MessageContentKind,
    MessageTemplateBindingsKind,
    MessageTemplateValueKind,
)

if TYPE_CHECKING:
    from .. import models as _models


class ActionBindings(_model_base.Model):
    """Binding actions to the interactive message.

    You probably want to use the sub-classes and not this class directly. Known sub-classes are:
    WhatsAppButtonActionBindings, WhatsAppListActionBindings, WhatsAppUrlActionBindings

    All required parameters must be populated in order to send to server.

    :ivar kind: Kind of the MessageActionBinding. Required. Known values are: "whatsAppListAction",
     "whatsAppButtonAction", and "whatsAppUrlAction".
    :vartype kind: str or ~azure.communication.messages.models.MessageActionBindingKind
    """

    __mapping__: Dict[str, _model_base.Model] = {}
    kind: str = rest_discriminator(name="kind")
    """Kind of the MessageActionBinding. Required. Known values are: \"whatsAppListAction\",
     \"whatsAppButtonAction\", and \"whatsAppUrlAction\"."""

    @overload
    def __init__(
        self,
        *,
        kind: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ActionGroup(_model_base.Model):
    """The Action Group content.

    All required parameters must be populated in order to send to server.

    :ivar title: Title of the ActionGroup. Required.
    :vartype title: str
    :ivar items_property: Array of items in ActionGroup. Required.
    :vartype items_property: list[~azure.communication.messages.models.ActionGroupItem]
    """

    title: str = rest_field()
    """Title of the ActionGroup. Required."""
    items_property: List["_models.ActionGroupItem"] = rest_field(name="items")
    """Array of items in ActionGroup. Required."""

    @overload
    def __init__(
        self,
        *,
        title: str,
        items_property: List["_models.ActionGroupItem"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class MessageContent(_model_base.Model):
    """The message content object used to create interactive messages components.

    You probably want to use the sub-classes and not this class directly. Known sub-classes are:
    ButtonSetContent, DocumentMessageContent, ActionGroupContent, ImageMessageContent,
    TextMessageContent, LinkContent, VideoMessageContent

    All required parameters must be populated in order to send to server.

    :ivar kind: Kind of MessageContent. Required. Known values are: "text", "image", "video",
     "document", "group", "buttonSet", and "url".
    :vartype kind: str or ~azure.communication.messages.models.MessageContentKind
    """

    __mapping__: Dict[str, _model_base.Model] = {}
    kind: str = rest_discriminator(name="kind")
    """Kind of MessageContent. Required. Known values are: \"text\", \"image\", \"video\",
     \"document\", \"group\", \"buttonSet\", and \"url\"."""

    @overload
    def __init__(
        self,
        *,
        kind: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ActionGroupContent(MessageContent, discriminator="group"):
    """The action content of type ActionGroup.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message content kind is actionGroup. Required. The ActionSet content type.
    :vartype kind: str or ~azure.communication.messages.models.GROUP
    :ivar title: Title of the actionGroup content. Required.
    :vartype title: str
    :ivar groups: Set or group of actions. Required.
    :vartype groups: list[~azure.communication.messages.models.ActionGroup]
    """

    kind: Literal[MessageContentKind.GROUP] = rest_discriminator(name="kind")  # type: ignore
    """Message content kind is actionGroup. Required. The ActionSet content type."""
    title: str = rest_field()
    """Title of the actionGroup content. Required."""
    groups: List["_models.ActionGroup"] = rest_field()
    """Set or group of actions. Required."""

    @overload
    def __init__(
        self,
        *,
        title: str,
        groups: List["_models.ActionGroup"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageContentKind.GROUP, **kwargs)


class ActionGroupItem(_model_base.Model):
    """The Action group item in the content.

    All required parameters must be populated in order to send to server.

    :ivar id: Id of the Item. Required.
    :vartype id: str
    :ivar title: Title of the Item. Required.
    :vartype title: str
    :ivar description: Description of the Item. Required.
    :vartype description: str
    """

    id: str = rest_field()
    """Id of the Item. Required."""
    title: str = rest_field()
    """Title of the Item. Required."""
    description: str = rest_field()
    """Description of the Item. Required."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        title: str,
        description: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class NotificationContent(_model_base.Model):
    """Details of the message to send.

    You probably want to use the sub-classes and not this class directly. Known sub-classes are:
    AudioNotificationContent, DocumentNotificationContent, ImageNotificationContent,
    MediaNotificationContent, InteractiveNotificationContent, ReactionNotificationContent,
    StickerNotificationContent, TemplateNotificationContent, TextNotificationContent,
    VideoNotificationContent

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: The type discriminator describing a notification type. Required. Known values are:
     "text", "image", "image_v0", "document", "video", "audio", "template", "sticker", "reaction",
     and "interactive".
    :vartype kind: str or ~azure.communication.messages.models.CommunicationMessageKind
    """

    __mapping__: Dict[str, _model_base.Model] = {}
    channel_registration_id: str = rest_field(name="channelRegistrationId")
    """The Channel Registration ID for the Business Identifier. Required."""
    to: List[str] = rest_field()
    """The native external platform user identifiers of the recipient. Required."""
    kind: str = rest_discriminator(name="kind")
    """The type discriminator describing a notification type. Required. Known values are: \"text\",
     \"image\", \"image_v0\", \"document\", \"video\", \"audio\", \"template\", \"sticker\",
     \"reaction\", and \"interactive\"."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        kind: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class AudioNotificationContent(NotificationContent, discriminator="audio"):
    """A request to send an audio notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is audio. Required. Audio message type.
    :vartype kind: str or ~azure.communication.messages.models.AUDIO
    :ivar media_uri: A media url for the file. Required if the type is one of the supported media
     types, e.g. image. Required.
    :vartype media_uri: str
    """

    kind: Literal[CommunicationMessageKind.AUDIO] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is audio. Required. Audio message type."""
    media_uri: str = rest_field(name="mediaUri")
    """A media url for the file. Required if the type is one of the supported media types, e.g. image.
     Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        media_uri: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.AUDIO, **kwargs)


class ButtonContent(_model_base.Model):
    """The message content of type Button information.

    All required parameters must be populated in order to send to server.

    :ivar id: Unique Id of the button content. Required.
    :vartype id: str
    :ivar title: Title of the button content. Required.
    :vartype title: str
    """

    id: str = rest_field()
    """Unique Id of the button content. Required."""
    title: str = rest_field()
    """Title of the button content. Required."""

    @overload
    def __init__(
        self,
        *,
        id: str,  # pylint: disable=redefined-builtin
        title: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ButtonSetContent(MessageContent, discriminator="buttonSet"):
    """The message content of type ButtonSet/ List of buttons information.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message content kind is Button. Required. The ButtonSet content type.
    :vartype kind: str or ~azure.communication.messages.models.BUTTON_SET
    :ivar buttons: Unique Id of the button content. Required.
    :vartype buttons: list[~azure.communication.messages.models.ButtonContent]
    """

    kind: Literal[MessageContentKind.BUTTON_SET] = rest_discriminator(name="kind")  # type: ignore
    """Message content kind is Button. Required. The ButtonSet content type."""
    buttons: List["_models.ButtonContent"] = rest_field()
    """Unique Id of the button content. Required."""

    @overload
    def __init__(
        self,
        *,
        buttons: List["_models.ButtonContent"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageContentKind.BUTTON_SET, **kwargs)


class DocumentMessageContent(MessageContent, discriminator="document"):
    """The message content of type document information.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message content kind is document. Required. The document content type.
    :vartype kind: str or ~azure.communication.messages.models.DOCUMENT
    :ivar media_uri: MediaUri of the media content. Required.
    :vartype media_uri: str
    """

    kind: Literal[MessageContentKind.DOCUMENT] = rest_discriminator(name="kind")  # type: ignore
    """Message content kind is document. Required. The document content type."""
    media_uri: str = rest_field(name="mediaUri")
    """MediaUri of the media content. Required."""

    @overload
    def __init__(
        self,
        *,
        media_uri: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageContentKind.DOCUMENT, **kwargs)


class DocumentNotificationContent(NotificationContent, discriminator="document"):
    """A request to send a document notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is image. Required. Document message type.
    :vartype kind: str or ~azure.communication.messages.models.DOCUMENT
    :ivar caption: Optional text content.
    :vartype caption: str
    :ivar file_name: Optional name for the file.
    :vartype file_name: str
    :ivar media_uri: A media url for the file. Required if the type is one of the supported media
     types, e.g. image. Required.
    :vartype media_uri: str
    """

    kind: Literal[CommunicationMessageKind.DOCUMENT] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is image. Required. Document message type."""
    caption: Optional[str] = rest_field()
    """Optional text content."""
    file_name: Optional[str] = rest_field(name="fileName")
    """Optional name for the file."""
    media_uri: str = rest_field(name="mediaUri")
    """A media url for the file. Required if the type is one of the supported media types, e.g. image.
     Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        media_uri: str,
        caption: Optional[str] = None,
        file_name: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.DOCUMENT, **kwargs)


class ImageMessageContent(MessageContent, discriminator="image"):
    """The message content of type image information.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message content kind is image. Required. The image content type.
    :vartype kind: str or ~azure.communication.messages.models.IMAGE
    :ivar media_uri: MediaUri of the media content. Required.
    :vartype media_uri: str
    """

    kind: Literal[MessageContentKind.IMAGE] = rest_discriminator(name="kind")  # type: ignore
    """Message content kind is image. Required. The image content type."""
    media_uri: str = rest_field(name="mediaUri")
    """MediaUri of the media content. Required."""

    @overload
    def __init__(
        self,
        *,
        media_uri: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageContentKind.IMAGE, **kwargs)


class ImageNotificationContent(NotificationContent, discriminator="image"):
    """A request to send an image notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is image. Required. Image message type.
    :vartype kind: str or ~azure.communication.messages.models.IMAGE
    :ivar content: Optional text content.
    :vartype content: str
    :ivar media_uri: A media url for the file. Required if the type is one of the supported media
     types, e.g. image. Required.
    :vartype media_uri: str
    """

    kind: Literal[CommunicationMessageKind.IMAGE] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is image. Required. Image message type."""
    content: Optional[str] = rest_field(name="caption")
    """Optional text content."""
    media_uri: str = rest_field(name="mediaUri")
    """A media url for the file. Required if the type is one of the supported media types, e.g. image.
     Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        media_uri: str,
        content: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.IMAGE, **kwargs)


class InteractiveMessage(_model_base.Model):
    """The Interactive message content to which user can read and respond.

    All required parameters must be populated in order to send to server.

    :ivar header: Gets or Sets Header content. Supports the following types:text, images etc.
    :vartype header: ~azure.communication.messages.models.MessageContent
    :ivar body: Gets or Sets Message body content. Emojis, markdown, and links are supported.
     Required.
    :vartype body: ~azure.communication.messages.models.TextMessageContent
    :ivar footer: Gets or Sets Message footer content. Emojis, markdown, and links are supported.
    :vartype footer: ~azure.communication.messages.models.TextMessageContent
    :ivar action: The binding object to get or set Action which describes options user have to
     respond to message. Required.
    :vartype action: ~azure.communication.messages.models.ActionBindings
    """

    header: Optional["_models.MessageContent"] = rest_field()
    """Gets or Sets Header content. Supports the following types:text, images etc."""
    body: "_models.TextMessageContent" = rest_field()
    """Gets or Sets Message body content. Emojis, markdown, and links are supported. Required."""
    footer: Optional["_models.TextMessageContent"] = rest_field()
    """Gets or Sets Message footer content. Emojis, markdown, and links are supported."""
    action: "_models.ActionBindings" = rest_field()
    """The binding object to get or set Action which describes options user have to respond to
     message. Required."""

    @overload
    def __init__(
        self,
        *,
        body: "_models.TextMessageContent",
        action: "_models.ActionBindings",
        header: Optional["_models.MessageContent"] = None,
        footer: Optional["_models.TextMessageContent"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class InteractiveNotificationContent(NotificationContent, discriminator="interactive"):
    """A request to send an Interactive message notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is Interactive. Required. Interactive Actionable message
     type.
    :vartype kind: str or ~azure.communication.messages.models.INTERACTIVE
    :ivar interactive_message: The interactive message content. Required.
    :vartype interactive_message: ~azure.communication.messages.models.InteractiveMessage
    """

    kind: Literal[CommunicationMessageKind.INTERACTIVE] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is Interactive. Required. Interactive Actionable message type."""
    interactive_message: "_models.InteractiveMessage" = rest_field(name="interactiveMessage")
    """The interactive message content. Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        interactive_message: "_models.InteractiveMessage",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.INTERACTIVE, **kwargs)


class LinkContent(MessageContent, discriminator="url"):
    """The message content of type Url information.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message content kind is url. Required. The Url content type.
    :vartype kind: str or ~azure.communication.messages.models.URL
    :ivar title: Title of the url content. Required.
    :vartype title: str
    :ivar url: The url in the content. Required.
    :vartype url: str
    """

    kind: Literal[MessageContentKind.URL] = rest_discriminator(name="kind")  # type: ignore
    """Message content kind is url. Required. The Url content type."""
    title: str = rest_field()
    """Title of the url content. Required."""
    url: str = rest_field()
    """The url in the content. Required."""

    @overload
    def __init__(
        self,
        *,
        title: str,
        url: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageContentKind.URL, **kwargs)


class MediaNotificationContent(NotificationContent, discriminator="image_v0"):
    """@deprecated A request to send an image notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is image. Required. Image message type. Legacy image type
     for ``MediaNotificationContent`` which is being deprecated
    :vartype kind: str or ~azure.communication.messages.models.IMAGE_V0
    :ivar content: Optional text content.
    :vartype content: str
    :ivar media_uri: A media url for the file. Required if the type is one of the supported media
     types, e.g. image. Required.
    :vartype media_uri: str
    """

    kind: Literal[CommunicationMessageKind.IMAGE_V0] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is image. Required. Image message type. Legacy image type for
     ``MediaNotificationContent`` which is being deprecated"""
    content: Optional[str] = rest_field()
    """Optional text content."""
    media_uri: str = rest_field(name="mediaUri")
    """A media url for the file. Required if the type is one of the supported media types, e.g. image.
     Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        media_uri: str,
        content: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.IMAGE_V0, **kwargs)


class MessageReceipt(_model_base.Model):
    """Receipt of the sending one message.


    :ivar message_id: The message id. Required.
    :vartype message_id: str
    :ivar to: The native external platform user identifier of the recipient. Required.
    :vartype to: str
    """

    message_id: str = rest_field(name="messageId")
    """The message id. Required."""
    to: str = rest_field()
    """The native external platform user identifier of the recipient. Required."""

    @overload
    def __init__(
        self,
        *,
        message_id: str,
        to: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class MessageTemplate(_model_base.Model):
    """The template object used to create templates.

    All required parameters must be populated in order to send to server.

    :ivar name: Name of the template. Required.
    :vartype name: str
    :ivar language: The template's language, in the ISO 639 format, consist of a two-letter
     language code followed by an optional two-letter country code, e.g., 'en' or 'en_US'. Required.
    :vartype language: str
    :ivar template_values: The template values.
    :vartype template_values: list[~azure.communication.messages.models.MessageTemplateValue]
    :ivar bindings: The binding object to link values to the template specific locations.
    :vartype bindings: ~azure.communication.messages.models.MessageTemplateBindings
    """

    name: str = rest_field()
    """Name of the template. Required."""
    language: str = rest_field()
    """The template's language, in the ISO 639 format, consist of a two-letter language code followed
     by an optional two-letter country code, e.g., 'en' or 'en_US'. Required."""
    template_values: Optional[List["_models.MessageTemplateValue"]] = rest_field(name="values")
    """The template values."""
    bindings: Optional["_models.MessageTemplateBindings"] = rest_field()
    """The binding object to link values to the template specific locations."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        language: str,
        template_values: Optional[List["_models.MessageTemplateValue"]] = None,
        bindings: Optional["_models.MessageTemplateBindings"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class MessageTemplateBindings(_model_base.Model):
    """The binding object to link values to the template specific locations.

    You probably want to use the sub-classes and not this class directly. Known sub-classes are:
    WhatsAppMessageTemplateBindings

    All required parameters must be populated in order to send to server.

    :ivar kind: The type discriminator describing a template bindings type. Required. "whatsApp"
    :vartype kind: str or ~azure.communication.messages.models.MessageTemplateBindingsKind
    """

    __mapping__: Dict[str, _model_base.Model] = {}
    kind: str = rest_discriminator(name="kind")
    """The type discriminator describing a template bindings type. Required. \"whatsApp\""""

    @overload
    def __init__(
        self,
        *,
        kind: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class MessageTemplateValue(_model_base.Model):
    """The class describes a parameter of a template.

    You probably want to use the sub-classes and not this class directly. Known sub-classes are:
    MessageTemplateDocument, MessageTemplateImage, MessageTemplateLocation,
    MessageTemplateQuickAction, MessageTemplateText, MessageTemplateVideo

    All required parameters must be populated in order to send to server.

    :ivar name: Template binding reference name. Required.
    :vartype name: str
    :ivar kind: The type discriminator describing a template parameter type. Required. Known values
     are: "text", "image", "document", "video", "location", and "quickAction".
    :vartype kind: str or ~azure.communication.messages.models.MessageTemplateValueKind
    """

    __mapping__: Dict[str, _model_base.Model] = {}
    name: str = rest_field()
    """Template binding reference name. Required."""
    kind: str = rest_discriminator(name="kind")
    """The type discriminator describing a template parameter type. Required. Known values are:
     \"text\", \"image\", \"document\", \"video\", \"location\", and \"quickAction\"."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        kind: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class MessageTemplateDocument(MessageTemplateValue, discriminator="document"):
    """The message template's document value information.

    All required parameters must be populated in order to send to server.

    :ivar name: Template binding reference name. Required.
    :vartype name: str
    :ivar kind: Message parameter type is document. Required. The document template parameter type.
    :vartype kind: str or ~azure.communication.messages.models.DOCUMENT
    :ivar url: The (public) URL of the media. Required.
    :vartype url: str
    :ivar caption: The [optional] caption of the media object.
    :vartype caption: str
    :ivar file_name: The [optional] filename of the media file.
    :vartype file_name: str
    """

    kind: Literal[MessageTemplateValueKind.DOCUMENT] = rest_discriminator(name="kind")  # type: ignore
    """Message parameter type is document. Required. The document template parameter type."""
    url: str = rest_field()
    """The (public) URL of the media. Required."""
    caption: Optional[str] = rest_field()
    """The [optional] caption of the media object."""
    file_name: Optional[str] = rest_field(name="fileName")
    """The [optional] filename of the media file."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        url: str,
        caption: Optional[str] = None,
        file_name: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageTemplateValueKind.DOCUMENT, **kwargs)


class MessageTemplateImage(MessageTemplateValue, discriminator="image"):
    """The message template's image value information.

    All required parameters must be populated in order to send to server.

    :ivar name: Template binding reference name. Required.
    :vartype name: str
    :ivar kind: Message parameter type is image. Required. The image template parameter type.
    :vartype kind: str or ~azure.communication.messages.models.IMAGE
    :ivar url: The (public) URL of the media. Required.
    :vartype url: str
    :ivar caption: The [optional] caption of the media object.
    :vartype caption: str
    :ivar file_name: The [optional] filename of the media file.
    :vartype file_name: str
    """

    kind: Literal[MessageTemplateValueKind.IMAGE] = rest_discriminator(name="kind")  # type: ignore
    """Message parameter type is image. Required. The image template parameter type."""
    url: str = rest_field()
    """The (public) URL of the media. Required."""
    caption: Optional[str] = rest_field()
    """The [optional] caption of the media object."""
    file_name: Optional[str] = rest_field(name="fileName")
    """The [optional] filename of the media file."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        url: str,
        caption: Optional[str] = None,
        file_name: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageTemplateValueKind.IMAGE, **kwargs)


class MessageTemplateItem(_model_base.Model):
    """The message template as returned from the service.

    You probably want to use the sub-classes and not this class directly. Known sub-classes are:
    WhatsAppMessageTemplateItem

    Readonly variables are only populated by the server, and will be ignored when sending a request.


    :ivar name: The template's name. Required.
    :vartype name: str
    :ivar language: The template's language, in the ISO 639 format, consist of a two-letter
     language code followed by an optional two-letter country code, e.g., 'en' or 'en_US'. Required.
    :vartype language: str
    :ivar status: The aggregated template status. Required. Known values are: "approved",
     "rejected", "pending", and "paused".
    :vartype status: str or ~azure.communication.messages.models.MessageTemplateStatus
    :ivar kind: The type discriminator describing a template type. Required. "whatsApp"
    :vartype kind: str or ~azure.communication.messages.models.CommunicationMessagesChannel
    """

    __mapping__: Dict[str, _model_base.Model] = {}
    name: str = rest_field(visibility=["read"])
    """The template's name. Required."""
    language: str = rest_field()
    """The template's language, in the ISO 639 format, consist of a two-letter language code followed
     by an optional two-letter country code, e.g., 'en' or 'en_US'. Required."""
    status: Union[str, "_models.MessageTemplateStatus"] = rest_field()
    """The aggregated template status. Required. Known values are: \"approved\", \"rejected\",
     \"pending\", and \"paused\"."""
    kind: str = rest_discriminator(name="kind")
    """The type discriminator describing a template type. Required. \"whatsApp\""""

    @overload
    def __init__(
        self,
        *,
        language: str,
        status: Union[str, "_models.MessageTemplateStatus"],
        kind: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class MessageTemplateLocation(MessageTemplateValue, discriminator="location"):
    """The message template's location value information.

    All required parameters must be populated in order to send to server.

    :ivar name: Template binding reference name. Required.
    :vartype name: str
    :ivar kind: Message parameter type is location. Required. The location template parameter type.
    :vartype kind: str or ~azure.communication.messages.models.LOCATION
    :ivar location_name: The [Optional] name of the location.
    :vartype location_name: str
    :ivar address: The [Optional] address of the location.
    :vartype address: str
    :ivar latitude: The latitude of the location. Required.
    :vartype latitude: float
    :ivar longitude: The longitude of the location. Required.
    :vartype longitude: float
    """

    kind: Literal[MessageTemplateValueKind.LOCATION] = rest_discriminator(name="kind")  # type: ignore
    """Message parameter type is location. Required. The location template parameter type."""
    location_name: Optional[str] = rest_field(name="locationName")
    """The [Optional] name of the location."""
    address: Optional[str] = rest_field()
    """The [Optional] address of the location."""
    latitude: float = rest_field()
    """The latitude of the location. Required."""
    longitude: float = rest_field()
    """The longitude of the location. Required."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        latitude: float,
        longitude: float,
        location_name: Optional[str] = None,
        address: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageTemplateValueKind.LOCATION, **kwargs)


class MessageTemplateQuickAction(MessageTemplateValue, discriminator="quickAction"):
    """The message template's quick action value information.

    All required parameters must be populated in order to send to server.

    :ivar name: Template binding reference name. Required.
    :vartype name: str
    :ivar kind: Message parameter type is quick action. Required. The quick action template
     parameter type.
    :vartype kind: str or ~azure.communication.messages.models.QUICK_ACTION
    :ivar text: The [Optional] quick action text.
    :vartype text: str
    :ivar payload: The [Optional] quick action payload.
    :vartype payload: str
    """

    kind: Literal[MessageTemplateValueKind.QUICK_ACTION] = rest_discriminator(name="kind")  # type: ignore
    """Message parameter type is quick action. Required. The quick action template parameter type."""
    text: Optional[str] = rest_field()
    """The [Optional] quick action text."""
    payload: Optional[str] = rest_field()
    """The [Optional] quick action payload."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        text: Optional[str] = None,
        payload: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageTemplateValueKind.QUICK_ACTION, **kwargs)


class MessageTemplateText(MessageTemplateValue, discriminator="text"):
    """The message template's text value information.

    All required parameters must be populated in order to send to server.

    :ivar name: Template binding reference name. Required.
    :vartype name: str
    :ivar kind: Message parameter type is text. Required. The text template parameter type.
    :vartype kind: str or ~azure.communication.messages.models.TEXT
    :ivar text: The text value. Required.
    :vartype text: str
    """

    kind: Literal[MessageTemplateValueKind.TEXT] = rest_discriminator(name="kind")  # type: ignore
    """Message parameter type is text. Required. The text template parameter type."""
    text: str = rest_field()
    """The text value. Required."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        text: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageTemplateValueKind.TEXT, **kwargs)


class MessageTemplateVideo(MessageTemplateValue, discriminator="video"):
    """The message template's video value information.

    All required parameters must be populated in order to send to server.

    :ivar name: Template binding reference name. Required.
    :vartype name: str
    :ivar kind: Message parameter type is video. Required. The video template parameter type.
    :vartype kind: str or ~azure.communication.messages.models.VIDEO
    :ivar url: The (public) URL of the media. Required.
    :vartype url: str
    :ivar caption: The [optional] caption of the media object.
    :vartype caption: str
    :ivar file_name: The [optional] filename of the media file.
    :vartype file_name: str
    """

    kind: Literal[MessageTemplateValueKind.VIDEO] = rest_discriminator(name="kind")  # type: ignore
    """Message parameter type is video. Required. The video template parameter type."""
    url: str = rest_field()
    """The (public) URL of the media. Required."""
    caption: Optional[str] = rest_field()
    """The [optional] caption of the media object."""
    file_name: Optional[str] = rest_field(name="fileName")
    """The [optional] filename of the media file."""

    @overload
    def __init__(
        self,
        *,
        name: str,
        url: str,
        caption: Optional[str] = None,
        file_name: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageTemplateValueKind.VIDEO, **kwargs)


class ReactionNotificationContent(NotificationContent, discriminator="reaction"):
    """A request to send a Reaction notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is reaction. Required. Reaction message type.
    :vartype kind: str or ~azure.communication.messages.models.REACTION
    :ivar emoji: emoji content like uD83D. Required.
    :vartype emoji: str
    :ivar message_id: ID of the previous message you want to reply to. Required.
    :vartype message_id: str
    """

    kind: Literal[CommunicationMessageKind.REACTION] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is reaction. Required. Reaction message type."""
    emoji: str = rest_field()
    """emoji content like :) Required."""
    message_id: str = rest_field(name="messageId")
    """ID of the previous message you want to reply to. Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        emoji: str,
        message_id: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.REACTION, **kwargs)


class SendMessageResult(_model_base.Model):
    """Result of the send message operation.


    :ivar receipts: Receipts of the send message operation. Required.
    :vartype receipts: list[~azure.communication.messages.models.MessageReceipt]
    """

    receipts: List["_models.MessageReceipt"] = rest_field()
    """Receipts of the send message operation. Required."""

    @overload
    def __init__(
        self,
        *,
        receipts: List["_models.MessageReceipt"],
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class StickerNotificationContent(NotificationContent, discriminator="sticker"):
    """A request to send a Sticker notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is sticker. Required. Sticker message type.
    :vartype kind: str or ~azure.communication.messages.models.STICKER
    :ivar media_uri: A media url for the file. Required if the type is one of the supported media
     types, e.g. image. Required.
    :vartype media_uri: str
    """

    kind: Literal[CommunicationMessageKind.STICKER] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is sticker. Required. Sticker message type."""
    media_uri: str = rest_field(name="mediaUri")
    """A media url for the file. Required if the type is one of the supported media types, e.g. image.
     Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        media_uri: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.STICKER, **kwargs)


class TemplateNotificationContent(NotificationContent, discriminator="template"):
    """A request to send a template notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is template. Required. Template message type.
    :vartype kind: str or ~azure.communication.messages.models.TEMPLATE
    :ivar template: The template object used to create templates. Required.
    :vartype template: ~azure.communication.messages.models.MessageTemplate
    """

    kind: Literal[CommunicationMessageKind.TEMPLATE] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is template. Required. Template message type."""
    template: "_models.MessageTemplate" = rest_field()
    """The template object used to create templates. Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        template: "_models.MessageTemplate",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.TEMPLATE, **kwargs)


class TextMessageContent(MessageContent, discriminator="text"):
    """The message content of type text information.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message content kind is text. Required. The text content type.
    :vartype kind: str or ~azure.communication.messages.models.TEXT
    :ivar text: The text value. Required.
    :vartype text: str
    """

    kind: Literal[MessageContentKind.TEXT] = rest_discriminator(name="kind")  # type: ignore
    """Message content kind is text. Required. The text content type."""
    text: str = rest_field()
    """The text value. Required."""

    @overload
    def __init__(
        self,
        *,
        text: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageContentKind.TEXT, **kwargs)


class TextNotificationContent(NotificationContent, discriminator="text"):
    """A request to send a text notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is text. Required. Text message type.
    :vartype kind: str or ~azure.communication.messages.models.TEXT
    :ivar content: Message content. Required.
    :vartype content: str
    """

    kind: Literal[CommunicationMessageKind.TEXT] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is text. Required. Text message type."""
    content: str = rest_field()
    """Message content. Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        content: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.TEXT, **kwargs)


class VideoMessageContent(MessageContent, discriminator="video"):
    """The message content of type video information.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message content kind is video. Required. The video content type.
    :vartype kind: str or ~azure.communication.messages.models.VIDEO
    :ivar media_uri: MediaUri of the media content. Required.
    :vartype media_uri: str
    """

    kind: Literal[MessageContentKind.VIDEO] = rest_discriminator(name="kind")  # type: ignore
    """Message content kind is video. Required. The video content type."""
    media_uri: str = rest_field(name="mediaUri")
    """MediaUri of the media content. Required."""

    @overload
    def __init__(
        self,
        *,
        media_uri: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageContentKind.VIDEO, **kwargs)


class VideoNotificationContent(NotificationContent, discriminator="video"):
    """A request to send a video notification.

    All required parameters must be populated in order to send to server.

    :ivar channel_registration_id: The Channel Registration ID for the Business Identifier.
     Required.
    :vartype channel_registration_id: str
    :ivar to: The native external platform user identifiers of the recipient. Required.
    :vartype to: list[str]
    :ivar kind: Message notification type is image. Required. Video message type.
    :vartype kind: str or ~azure.communication.messages.models.VIDEO
    :ivar caption: Optional text content.
    :vartype caption: str
    :ivar media_uri: A media url for the file. Required if the type is one of the supported media
     types, e.g. image. Required.
    :vartype media_uri: str
    """

    kind: Literal[CommunicationMessageKind.VIDEO] = rest_discriminator(name="kind")  # type: ignore
    """Message notification type is image. Required. Video message type."""
    caption: Optional[str] = rest_field()
    """Optional text content."""
    media_uri: str = rest_field(name="mediaUri")
    """A media url for the file. Required if the type is one of the supported media types, e.g. image.
     Required."""

    @overload
    def __init__(
        self,
        *,
        channel_registration_id: str,
        to: List[str],
        media_uri: str,
        caption: Optional[str] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessageKind.VIDEO, **kwargs)


class WhatsAppButtonActionBindings(ActionBindings, discriminator="whatsAppButtonAction"):
    """WhatsApp Binding actions to the interactive message.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message action binding type is WhatsAppButtonAction. Required. The WhatsApp Button
     action binding kind.
    :vartype kind: str or ~azure.communication.messages.models.WHATS_APP_BUTTON_ACTION
    :ivar content: Action content of Interactive message. Required.
    :vartype content: ~azure.communication.messages.models.ButtonSetContent
    """

    kind: Literal[MessageActionBindingKind.WHATS_APP_BUTTON_ACTION] = rest_discriminator(name="kind")  # type: ignore
    """Message action binding type is WhatsAppButtonAction. Required. The WhatsApp Button action
     binding kind."""
    content: "_models.ButtonSetContent" = rest_field()
    """Action content of Interactive message. Required."""

    @overload
    def __init__(
        self,
        *,
        content: "_models.ButtonSetContent",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageActionBindingKind.WHATS_APP_BUTTON_ACTION, **kwargs)


class WhatsAppListActionBindings(ActionBindings, discriminator="whatsAppListAction"):
    """WhatsApp List Binding actions to the interactive message.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message action binding type is WhatsAppListAction. Required. The WhatsApp List
     action binding kind.
    :vartype kind: str or ~azure.communication.messages.models.WHATS_APP_LIST_ACTION
    :ivar content: Action content of Interactive message. Required.
    :vartype content: ~azure.communication.messages.models.ActionGroupContent
    """

    kind: Literal[MessageActionBindingKind.WHATS_APP_LIST_ACTION] = rest_discriminator(name="kind")  # type: ignore
    """Message action binding type is WhatsAppListAction. Required. The WhatsApp List action binding
     kind."""
    content: "_models.ActionGroupContent" = rest_field()
    """Action content of Interactive message. Required."""

    @overload
    def __init__(
        self,
        *,
        content: "_models.ActionGroupContent",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageActionBindingKind.WHATS_APP_LIST_ACTION, **kwargs)


class WhatsAppMessageTemplateBindings(MessageTemplateBindings, discriminator="whatsApp"):
    """The template bindings for WhatsApp.

    All required parameters must be populated in order to send to server.

    :ivar kind: MessageTemplateBindings is whatsApp. Required. The WhatsApp template type.
    :vartype kind: str or ~azure.communication.messages.models.WHATSAPP
    :ivar header: The header template bindings.
    :vartype header:
     list[~azure.communication.messages.models.WhatsAppMessageTemplateBindingsComponent]
    :ivar body: The body template bindings.
    :vartype body:
     list[~azure.communication.messages.models.WhatsAppMessageTemplateBindingsComponent]
    :ivar footer: The footer template bindings.
    :vartype footer:
     list[~azure.communication.messages.models.WhatsAppMessageTemplateBindingsComponent]
    :ivar buttons: The button template bindings.
    :vartype buttons:
     list[~azure.communication.messages.models.WhatsAppMessageTemplateBindingsButton]
    """

    kind: Literal[MessageTemplateBindingsKind.WHATSAPP] = rest_discriminator(name="kind")  # type: ignore
    """MessageTemplateBindings is whatsApp. Required. The WhatsApp template type."""
    header: Optional[List["_models.WhatsAppMessageTemplateBindingsComponent"]] = rest_field()
    """The header template bindings."""
    body: Optional[List["_models.WhatsAppMessageTemplateBindingsComponent"]] = rest_field()
    """The body template bindings."""
    footer: Optional[List["_models.WhatsAppMessageTemplateBindingsComponent"]] = rest_field()
    """The footer template bindings."""
    buttons: Optional[List["_models.WhatsAppMessageTemplateBindingsButton"]] = rest_field()
    """The button template bindings."""

    @overload
    def __init__(
        self,
        *,
        header: Optional[List["_models.WhatsAppMessageTemplateBindingsComponent"]] = None,
        body: Optional[List["_models.WhatsAppMessageTemplateBindingsComponent"]] = None,
        footer: Optional[List["_models.WhatsAppMessageTemplateBindingsComponent"]] = None,
        buttons: Optional[List["_models.WhatsAppMessageTemplateBindingsButton"]] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageTemplateBindingsKind.WHATSAPP, **kwargs)


class WhatsAppMessageTemplateBindingsButton(_model_base.Model):
    """The template bindings component button for WhatsApp.

    All required parameters must be populated in order to send to server.

    :ivar sub_type: The WhatsApp button sub type. Required. Known values are: "quickReply" and
     "url".
    :vartype sub_type: str or ~azure.communication.messages.models.WhatsAppMessageButtonSubType
    :ivar ref_value: The name of the referenced item in the template values. Required.
    :vartype ref_value: str
    """

    sub_type: Union[str, "_models.WhatsAppMessageButtonSubType"] = rest_field(name="subType")
    """The WhatsApp button sub type. Required. Known values are: \"quickReply\" and \"url\"."""
    ref_value: str = rest_field(name="refValue")
    """The name of the referenced item in the template values. Required."""

    @overload
    def __init__(
        self,
        *,
        sub_type: Union[str, "_models.WhatsAppMessageButtonSubType"],
        ref_value: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class WhatsAppMessageTemplateBindingsComponent(_model_base.Model):
    """The template bindings component for WhatsApp.

    All required parameters must be populated in order to send to server.

    :ivar ref_value: The name of the referenced item in the template values. Required.
    :vartype ref_value: str
    """

    ref_value: str = rest_field(name="refValue")
    """The name of the referenced item in the template values. Required."""

    @overload
    def __init__(
        self,
        *,
        ref_value: str,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class WhatsAppMessageTemplateItem(MessageTemplateItem, discriminator="whatsApp"):
    """The WhatsApp-specific template response contract.

    Readonly variables are only populated by the server, and will be ignored when sending a request.


    :ivar name: The template's name. Required.
    :vartype name: str
    :ivar language: The template's language, in the ISO 639 format, consist of a two-letter
     language code followed by an optional two-letter country code, e.g., 'en' or 'en_US'. Required.
    :vartype language: str
    :ivar status: The aggregated template status. Required. Known values are: "approved",
     "rejected", "pending", and "paused".
    :vartype status: str or ~azure.communication.messages.models.MessageTemplateStatus
    :ivar content: WhatsApp platform's template content. This is the payload returned from WhatsApp
     API.
    :vartype content: any
    :ivar kind: Message template response type is whatsApp. Required. The WhatsApp communication
     messages channel type.
    :vartype kind: str or ~azure.communication.messages.models.WHATSAPP
    """

    content: Optional[Any] = rest_field()
    """WhatsApp platform's template content. This is the payload returned from WhatsApp API."""
    kind: Literal[CommunicationMessagesChannel.WHATSAPP] = rest_discriminator(name="kind")  # type: ignore
    """Message template response type is whatsApp. Required. The WhatsApp communication messages
     channel type."""

    @overload
    def __init__(
        self,
        *,
        language: str,
        status: Union[str, "_models.MessageTemplateStatus"],
        content: Optional[Any] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=CommunicationMessagesChannel.WHATSAPP, **kwargs)


class WhatsAppUrlActionBindings(ActionBindings, discriminator="whatsAppUrlAction"):
    """WhatsApp Binding actions to the interactive message.

    All required parameters must be populated in order to send to server.

    :ivar kind: Message action binding type is WhatsAppUrlAction. Required. The WhatsApp Url action
     binding kind.
    :vartype kind: str or ~azure.communication.messages.models.WHATS_APP_URL_ACTION
    :ivar content: Action content of Interactive message. Required.
    :vartype content: ~azure.communication.messages.models.LinkContent
    """

    kind: Literal[MessageActionBindingKind.WHATS_APP_URL_ACTION] = rest_discriminator(name="kind")  # type: ignore
    """Message action binding type is WhatsAppUrlAction. Required. The WhatsApp Url action binding
     kind."""
    content: "_models.LinkContent" = rest_field()
    """Action content of Interactive message. Required."""

    @overload
    def __init__(
        self,
        *,
        content: "_models.LinkContent",
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, kind=MessageActionBindingKind.WHATS_APP_URL_ACTION, **kwargs)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import datetime
from typing import List, Optional, Union

from azure.core.exceptions import HttpResponseError
import msrest.serialization

from ._azure_change_analysis_management_client_enums import *


class Resource(msrest.serialization.Model):
    """Common fields that are returned in the response for all Azure Resource Manager resources.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(Resource, self).__init__(**kwargs)
        self.id = None
        self.name = None
        self.type = None


class Change(Resource):
    """The detected change.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :param properties: The properties of a change.
    :type properties: ~Microsoft.ChangeAnalysis.models.ChangeProperties
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'properties': {'key': 'properties', 'type': 'ChangeProperties'},
    }

    def __init__(
        self,
        *,
        properties: Optional["ChangeProperties"] = None,
        **kwargs
    ):
        super(Change, self).__init__(**kwargs)
        self.properties = properties


class ChangeList(msrest.serialization.Model):
    """The list of detected changes.

    :param value: The list of changes.
    :type value: list[~Microsoft.ChangeAnalysis.models.Change]
    :param next_link: The URI that can be used to request the next page of changes.
    :type next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[Change]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["Change"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        super(ChangeList, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link


class ChangeProperties(msrest.serialization.Model):
    """The properties of a change.

    :param resource_id: The resource id that the change is attached to.
    :type resource_id: str
    :param time_stamp: The time when the change is detected.
    :type time_stamp: ~datetime.datetime
    :param initiated_by_list: The list of identities who might initiated the change.
     The identity could be user name (email address) or the object ID of the Service Principal.
    :type initiated_by_list: list[str]
    :param change_type: The type of the change. Possible values include: "Add", "Remove", "Update".
    :type change_type: str or ~Microsoft.ChangeAnalysis.models.ChangeType
    :param property_changes: The list of detailed changes at json property level.
    :type property_changes: list[~Microsoft.ChangeAnalysis.models.PropertyChange]
    """

    _attribute_map = {
        'resource_id': {'key': 'resourceId', 'type': 'str'},
        'time_stamp': {'key': 'timeStamp', 'type': 'iso-8601'},
        'initiated_by_list': {'key': 'initiatedByList', 'type': '[str]'},
        'change_type': {'key': 'changeType', 'type': 'str'},
        'property_changes': {'key': 'propertyChanges', 'type': '[PropertyChange]'},
    }

    def __init__(
        self,
        *,
        resource_id: Optional[str] = None,
        time_stamp: Optional[datetime.datetime] = None,
        initiated_by_list: Optional[List[str]] = None,
        change_type: Optional[Union[str, "ChangeType"]] = None,
        property_changes: Optional[List["PropertyChange"]] = None,
        **kwargs
    ):
        super(ChangeProperties, self).__init__(**kwargs)
        self.resource_id = resource_id
        self.time_stamp = time_stamp
        self.initiated_by_list = initiated_by_list
        self.change_type = change_type
        self.property_changes = property_changes


class ErrorAdditionalInfo(msrest.serialization.Model):
    """The resource management error additional info.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar type: The additional info type.
    :vartype type: str
    :ivar info: The additional info.
    :vartype info: object
    """

    _validation = {
        'type': {'readonly': True},
        'info': {'readonly': True},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'info': {'key': 'info', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(ErrorAdditionalInfo, self).__init__(**kwargs)
        self.type = None
        self.info = None


class ErrorDetail(msrest.serialization.Model):
    """The error detail.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar code: The error code.
    :vartype code: str
    :ivar message: The error message.
    :vartype message: str
    :ivar target: The error target.
    :vartype target: str
    :ivar details: The error details.
    :vartype details: list[~Microsoft.ChangeAnalysis.models.ErrorDetail]
    :ivar additional_info: The error additional info.
    :vartype additional_info: list[~Microsoft.ChangeAnalysis.models.ErrorAdditionalInfo]
    """

    _validation = {
        'code': {'readonly': True},
        'message': {'readonly': True},
        'target': {'readonly': True},
        'details': {'readonly': True},
        'additional_info': {'readonly': True},
    }

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
        'target': {'key': 'target', 'type': 'str'},
        'details': {'key': 'details', 'type': '[ErrorDetail]'},
        'additional_info': {'key': 'additionalInfo', 'type': '[ErrorAdditionalInfo]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(ErrorDetail, self).__init__(**kwargs)
        self.code = None
        self.message = None
        self.target = None
        self.details = None
        self.additional_info = None


class ErrorResponse(msrest.serialization.Model):
    """Common error response for all Azure Resource Manager APIs to return error details for failed operations. (This also follows the OData error response format.).

    :param error: The error object.
    :type error: ~Microsoft.ChangeAnalysis.models.ErrorDetail
    """

    _attribute_map = {
        'error': {'key': 'error', 'type': 'ErrorDetail'},
    }

    def __init__(
        self,
        *,
        error: Optional["ErrorDetail"] = None,
        **kwargs
    ):
        super(ErrorResponse, self).__init__(**kwargs)
        self.error = error


class PropertyChange(msrest.serialization.Model):
    """Data of a property change.

    :param change_type: The type of the change. Possible values include: "Add", "Remove", "Update".
    :type change_type: str or ~Microsoft.ChangeAnalysis.models.ChangeType
    :param change_category: The change category. Possible values include: "User", "System".
    :type change_category: str or ~Microsoft.ChangeAnalysis.models.ChangeCategory
    :param json_path: The json path of the changed property.
    :type json_path: str
    :param display_name: The enhanced display name of the json path. E.g., the json path
     value[0].properties will be translated to something meaningful like
     slots["Staging"].properties.
    :type display_name: str
    :param level:  Possible values include: "Noisy", "Normal", "Important".
    :type level: str or ~Microsoft.ChangeAnalysis.models.Level
    :param description: The description of the changed property.
    :type description: str
    :param old_value: The value of the property before the change.
    :type old_value: str
    :param new_value: The value of the property after the change.
    :type new_value: str
    :param is_data_masked: The boolean indicating whether the oldValue and newValue are masked. The
     values are masked if it contains sensitive information that the user doesn't have access to.
    :type is_data_masked: bool
    """

    _attribute_map = {
        'change_type': {'key': 'changeType', 'type': 'str'},
        'change_category': {'key': 'changeCategory', 'type': 'str'},
        'json_path': {'key': 'jsonPath', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'level': {'key': 'level', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'old_value': {'key': 'oldValue', 'type': 'str'},
        'new_value': {'key': 'newValue', 'type': 'str'},
        'is_data_masked': {'key': 'isDataMasked', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        change_type: Optional[Union[str, "ChangeType"]] = None,
        change_category: Optional[Union[str, "ChangeCategory"]] = None,
        json_path: Optional[str] = None,
        display_name: Optional[str] = None,
        level: Optional[Union[str, "Level"]] = None,
        description: Optional[str] = None,
        old_value: Optional[str] = None,
        new_value: Optional[str] = None,
        is_data_masked: Optional[bool] = None,
        **kwargs
    ):
        super(PropertyChange, self).__init__(**kwargs)
        self.change_type = change_type
        self.change_category = change_category
        self.json_path = json_path
        self.display_name = display_name
        self.level = level
        self.description = description
        self.old_value = old_value
        self.new_value = new_value
        self.is_data_masked = is_data_masked


class ProxyResource(Resource):
    """The resource model definition for a Azure Resource Manager proxy resource. It will not have tags and a location.

    Variables are only populated by the server, and will be ignored when sending a request.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    """

    _validation = {
        'id': {'readonly': True},
        'name': {'readonly': True},
        'type': {'readonly': True},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(ProxyResource, self).__init__(**kwargs)


class ResourceProviderOperationDefinition(msrest.serialization.Model):
    """The resource provider operation definition.

    :param name: The resource provider operation name.
    :type name: str
    :param display: The resource provider operation details.
    :type display: ~Microsoft.ChangeAnalysis.models.ResourceProviderOperationDisplay
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'display': {'key': 'display', 'type': 'ResourceProviderOperationDisplay'},
    }

    def __init__(
        self,
        *,
        name: Optional[str] = None,
        display: Optional["ResourceProviderOperationDisplay"] = None,
        **kwargs
    ):
        super(ResourceProviderOperationDefinition, self).__init__(**kwargs)
        self.name = name
        self.display = display


class ResourceProviderOperationDisplay(msrest.serialization.Model):
    """The resource provider operation details.

    :param provider: Name of the resource provider.
    :type provider: str
    :param resource: Name of the resource type.
    :type resource: str
    :param operation: Name of the resource provider operation.
    :type operation: str
    :param description: Description of the resource provider operation.
    :type description: str
    """

    _attribute_map = {
        'provider': {'key': 'provider', 'type': 'str'},
        'resource': {'key': 'resource', 'type': 'str'},
        'operation': {'key': 'operation', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        provider: Optional[str] = None,
        resource: Optional[str] = None,
        operation: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs
    ):
        super(ResourceProviderOperationDisplay, self).__init__(**kwargs)
        self.provider = provider
        self.resource = resource
        self.operation = operation
        self.description = description


class ResourceProviderOperationList(msrest.serialization.Model):
    """The resource provider operation list.

    :param value: Resource provider operations list.
    :type value: list[~Microsoft.ChangeAnalysis.models.ResourceProviderOperationDefinition]
    :param next_link: The URI that can be used to request the next page for list of Azure
     operations.
    :type next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[ResourceProviderOperationDefinition]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        value: Optional[List["ResourceProviderOperationDefinition"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        super(ResourceProviderOperationList, self).__init__(**kwargs)
        self.value = value
        self.next_link = next_link



# The MIT License (MIT)
# Copyright (c) Microsoft Corporation. All rights reserved.

import collections
import os
import time
import unittest
import uuid

from azure.cosmos._retry_utility import _has_database_account_header, _has_read_retryable_headers, _configure_timeout
from azure.cosmos.cosmos_client import CosmosClient
from azure.cosmos.exceptions import CosmosHttpResponseError
from azure.cosmos.http_constants import StatusCodes
from azure.cosmos.partition_key import PartitionKey
from azure.cosmos import (ContainerProxy, DatabaseProxy, documents, exceptions,
                          http_constants, _retry_utility)
from azure.core.exceptions import AzureError, ServiceRequestError, ServiceResponseError, ClientAuthenticationError
from azure.core.pipeline.policies import AsyncRetryPolicy, RetryPolicy
from devtools_testutils.azure_recorded_testcase import get_credential
from devtools_testutils.helpers import is_live

try:
    import urllib3

    urllib3.disable_warnings()
except:
    print("no urllib3")


class TestConfig(object):
    local_host = 'https://localhost:8081/'
    # [SuppressMessage("Microsoft.Security", "CS002:SecretInNextLine", Justification="Cosmos DB Emulator Key")]
    masterKey = os.getenv('ACCOUNT_KEY',
                          'C2y6yDjf5/R+ob0N8A7Cgv30VRDJIWEHLM+4QDU5DE2nQ9nDuVTqobD4b8mGGyPMbIZnqyMsEcaGQy67XIw/Jw==')
    host = os.getenv('ACCOUNT_HOST', local_host)
    connection_str = os.getenv('ACCOUNT_CONNECTION_STR', 'AccountEndpoint={};AccountKey={};'.format(host, masterKey))

    connectionPolicy = documents.ConnectionPolicy()
    connectionPolicy.DisableSSLVerification = True
    is_emulator = host == local_host
    is_live._cache = True if not is_emulator else False
    credential = masterKey if is_emulator else get_credential()
    credential_async = masterKey if is_emulator else get_credential(is_async=True)

    global_host = os.getenv('GLOBAL_ACCOUNT_HOST', host)
    write_location_host = os.getenv('WRITE_LOCATION_HOST', host)
    read_location_host = os.getenv('READ_LOCATION_HOST', host)
    read_location2_host = os.getenv('READ_LOCATION_HOST2', host)
    global_masterKey = os.getenv('GLOBAL_ACCOUNT_KEY', masterKey)

    write_location = os.getenv('WRITE_LOCATION', host)
    read_location = os.getenv('READ_LOCATION', host)
    read_location2 = os.getenv('READ_LOCATION2', host)

    THROUGHPUT_FOR_5_PARTITIONS = 30000
    THROUGHPUT_FOR_2_PARTITIONS = 12000
    THROUGHPUT_FOR_1_PARTITION = 400

    TEST_DATABASE_ID = os.getenv('COSMOS_TEST_DATABASE_ID', "Python SDK Test Database " + str(uuid.uuid4()))

    TEST_SINGLE_PARTITION_CONTAINER_ID = "Single Partition Test Container " + str(uuid.uuid4())
    TEST_MULTI_PARTITION_CONTAINER_ID = "Multi Partition Test Container " + str(uuid.uuid4())
    TEST_SINGLE_PARTITION_PREFIX_PK_CONTAINER_ID = "Single Partition With Prefix PK Test Container " + str(uuid.uuid4())
    TEST_MULTI_PARTITION_PREFIX_PK_CONTAINER_ID = "Multi Partition With Prefix PK Test Container " + str(uuid.uuid4())

    TEST_CONTAINER_PARTITION_KEY = "pk"
    TEST_CONTAINER_PREFIX_PARTITION_KEY = ["pk1", "pk2"]
    TEST_CONTAINER_PREFIX_PARTITION_KEY_PATH = ['/pk1', '/pk2']

    @classmethod
    def create_database_if_not_exist(cls, client):
        # type: (CosmosClient) -> DatabaseProxy
        test_database = client.create_database_if_not_exists(cls.TEST_DATABASE_ID,
                                                             offer_throughput=cls.THROUGHPUT_FOR_1_PARTITION)
        return test_database

    @classmethod
    def create_single_partition_container_if_not_exist(cls, client):
        # type: (CosmosClient) -> ContainerProxy
        database = cls.create_database_if_not_exist(client)
        document_collection = database.create_container_if_not_exists(
            id=cls.TEST_SINGLE_PARTITION_CONTAINER_ID,
            partition_key=PartitionKey(path='/' + cls.TEST_CONTAINER_PARTITION_KEY, kind='Hash'),
            offer_throughput=cls.THROUGHPUT_FOR_1_PARTITION)
        return document_collection

    @classmethod
    def create_multi_partition_container_if_not_exist(cls, client):
        # type: (CosmosClient) -> ContainerProxy
        database = cls.create_database_if_not_exist(client)
        document_collection = database.create_container_if_not_exists(
            id=cls.TEST_MULTI_PARTITION_CONTAINER_ID,
            partition_key=PartitionKey(path='/' + cls.TEST_CONTAINER_PARTITION_KEY, kind='Hash'),
            offer_throughput=cls.THROUGHPUT_FOR_5_PARTITIONS)
        return document_collection

    @classmethod
    def create_single_partition_prefix_pk_container_if_not_exist(cls, client):
        # type: (CosmosClient) -> ContainerProxy
        database = cls.create_database_if_not_exist(client)
        document_collection = database.create_container_if_not_exists(
            id=cls.TEST_SINGLE_PARTITION_PREFIX_PK_CONTAINER_ID,
            partition_key=PartitionKey(path=cls.TEST_CONTAINER_PREFIX_PARTITION_KEY_PATH, kind='MultiHash'),
            offer_throughput=cls.THROUGHPUT_FOR_1_PARTITION)
        return document_collection

    @classmethod
    def create_multi_partition_prefix_pk_container_if_not_exist(cls, client):
        # type: (CosmosClient) -> ContainerProxy
        database = cls.create_database_if_not_exist(client)
        document_collection = database.create_container_if_not_exists(
            id=cls.TEST_MULTI_PARTITION_PREFIX_PK_CONTAINER_ID,
            partition_key=PartitionKey(path=cls.TEST_CONTAINER_PREFIX_PARTITION_KEY_PATH, kind='MultiHash'),
            offer_throughput=cls.THROUGHPUT_FOR_5_PARTITIONS)
        return document_collection

    @classmethod
    def try_delete_database(cls, client):
        # type: (CosmosClient) -> None
        try:
            client.delete_database(cls.TEST_DATABASE_ID)
        except exceptions.CosmosHttpResponseError as e:
            if e.status_code != StatusCodes.NOT_FOUND:
                raise e

    @classmethod
    def try_delete_database_with_id(cls, client, database_id):
        # type: (CosmosClient, str) -> None
        try:
            client.delete_database(database_id)
        except exceptions.CosmosHttpResponseError as e:
            if e.status_code != StatusCodes.NOT_FOUND:
                raise e

    @classmethod
    async def _validate_distinct_on_different_types_and_field_orders(cls, collection, query, expected_results):
        query_iterable = collection.query_items(query)
        results = [item async for item in query_iterable]
        for i in range(len(expected_results)):
            assert results[i] in expected_results

    @classmethod
    def _get_query_result_string(cls, query_result, fields):
        if type(query_result) is not dict:
            return str(query_result)
        res = str(query_result[fields[0]] if fields[0] in query_result else None)
        if len(fields) == 2:
            res = res + "," + str(query_result[fields[1]] if fields[1] in query_result else None)

        return res

    @classmethod
    async def _validate_distinct(cls, created_collection, query, results, is_select, fields):
        query_iterable = created_collection.query_items(query=query)
        query_results = [item async for item in query_iterable]

        assert len(results) == len(query_results)
        query_results_strings = []
        result_strings = []
        for i in range(len(results)):
            query_results_strings.append(cls._get_query_result_string(query_results[i], fields))
            result_strings.append(str(results[i]))
        if is_select:
            query_results_strings = sorted(query_results_strings)
            result_strings = sorted(result_strings)
        assert result_strings == query_results_strings

    @classmethod
    def _pad_with_none(cls, documents_param, field):
        for doc in documents_param:
            if field not in doc:
                doc[field] = None
        return documents_param

    @classmethod
    def _get_distinct_docs(cls, documents_param, field1, field2, is_order_by_or_value):
        if field2 is None:
            res = collections.OrderedDict.fromkeys(doc[field1] for doc in documents_param)
            if is_order_by_or_value:
                res = filter(lambda x: False if x is None else True, res)
        else:
            res = collections.OrderedDict.fromkeys(str(doc[field1]) + "," + str(doc[field2]) for doc in documents_param)
        return list(res)

    @classmethod
    def _get_order_by_docs(cls, documents_param, field1, field2):
        if field2 is None:
            return sorted(documents_param, key=lambda d: (d[field1] is not None, d[field1]))
        else:
            return sorted(documents_param,
                          key=lambda d: (d[field1] is not None, d[field1], d[field2] is not None, d[field2]))

    @classmethod
    async def _validate_distinct_offset_limit(cls, created_collection, query, results):
        query_iterable = created_collection.query_items(query=query)
        assert list(map(lambda doc: doc['value'], [item async for item in query_iterable])) == results

    @classmethod
    async def _validate_offset_limit(cls, created_collection, query, results):
        query_iterable = created_collection.query_items(query=query)
        assert list(map(lambda doc: doc['pk'], [item async for item in query_iterable])) == results

    @staticmethod
    def trigger_split(container, throughput):
        print("Triggering a split in session token helpers")
        container.replace_throughput(throughput)
        print("changed offer to 11k")
        print("--------------------------------")
        print("Waiting for split to complete")
        start_time = time.time()

        while True:
            offer = container.get_throughput()
            if offer.properties['content'].get('isOfferReplacePending', False):
                if time.time() - start_time > 60 * 25:  # timeout test at 25 minutes
                    unittest.skip("Partition split didn't complete in time.")
                else:
                    print("Waiting for split to complete")
                    time.sleep(60)
            else:
                break
        print("Split in session token helpers has completed")

    @staticmethod
    async def trigger_split_async(container, throughput):
        print("Triggering a split in session token helpers")
        await container.replace_throughput(throughput)
        print("changed offer to 11k")
        print("--------------------------------")
        print("Waiting for split to complete")
        start_time = time.time()

        while True:
            offer = await container.get_throughput()
            if offer.properties['content'].get('isOfferReplacePending', False):
                if time.time() - start_time > 60 * 25:  # timeout test at 25 minutes
                    unittest.skip("Partition split didn't complete in time.")
                else:
                    print("Waiting for split to complete")
                    time.sleep(60)
            else:
                break
        print("Split in session token helpers has completed")


def get_vector_indexing_policy(embedding_type):
    return {
        "indexingMode": "consistent",
        "includedPaths": [{"path": "/*"}],
        'excludedPaths': [{'path': '/"_etag"/?'}],
        "vectorIndexes": [
            {"path": "/embedding", "type": f"{embedding_type}"}
        ]
    }


def get_vector_embedding_policy(distance_function, data_type, dimensions):
    return {
        "vectorEmbeddings": [
            {
                "path": "/embedding",
                "dataType": f"{data_type}",
                "dimensions": dimensions,
                "distanceFunction": f"{distance_function}"
            }
        ]
    }


def get_full_text_indexing_policy(path):
    return {
        "indexingMode": "consistent",
        "includedPaths": [{"path": "/*"}],
        'excludedPaths': [{'path': '/"_etag"/?'}],
        "fullTextIndexes": [
            {"path": path}
        ]
    }


def get_full_text_policy(path):
    return {
        "defaultLanguage": "en-US",
        "fullTextPaths": [
            {
                "path": path,
                "language": "en-US"
            }
        ]
    }

class ResponseHookCaller:
    def __init__(self):
        self.count = 0

    def __call__(self, *args, **kwargs):
        self.count += 1


class FakeResponse:
    def __init__(self, headers):
        self.headers = headers
        self.reason = "foo"
        self.status_code = "bar"


class FakePipelineResponse:
    def __init__(self, headers=None, status_code=200, message="test-message"):
        self.http_response = FakeHttpResponse(headers, status_code, message)


class FakeHttpResponse:
    def __init__(self, headers=None, status_code=200, message="test-message"):
        if headers is None:
            headers = {}
        self.headers = headers
        self.status_code = status_code
        self.reason = message

    def body(self):
        return None


class MockConnectionRetryPolicy(RetryPolicy):
    def __init__(self, resource_type, error=None, **kwargs):
        self.resource_type = resource_type
        self.error = error
        self.counter = 0
        self.request_endpoints = []
        clean_kwargs = {k: v for k, v in kwargs.items() if v is not None}
        super().__init__(**clean_kwargs)

    def send(self, request):
        self.counter = 0
        absolute_timeout = request.context.options.pop('timeout', None)
        per_request_timeout = request.context.options.pop('connection_timeout', 0)
        request_params = request.context.options.pop('request_params', None)
        global_endpoint_manager = request.context.options.pop('global_endpoint_manager', None)
        retry_error = None
        retry_active = True
        response = None
        retry_settings = self.configure_retries(request.context.options)
        while retry_active:
            start_time = time.time()
            try:
                # raise the passed in exception for the passed in resource + operation combination
                if request.http_request.headers.get(http_constants.HttpHeaders.ThinClientProxyResourceType) == self.resource_type:
                    self.request_endpoints.append(request.http_request.url)
                    if self.error:
                        raise self.error
                _configure_timeout(request, absolute_timeout, per_request_timeout)
                response = self.next.send(request)
                break
            except ClientAuthenticationError:  # pylint:disable=try-except-raise
                # the authentication policy failed such that the client's request can't
                # succeed--we'll never have a response to it, so propagate the exception
                raise
            except exceptions.CosmosClientTimeoutError as timeout_error:
                timeout_error.inner_exception = retry_error
                timeout_error.response = response
                timeout_error.history = retry_settings['history']
                raise
            except ServiceRequestError as err:
                retry_error = err
                # the request ran into a socket timeout or failed to establish a new connection
                # since request wasn't sent, raise exception immediately to be dealt with in client retry policies
                # This logic is based on the _retry.py file from azure-core
                if (not _has_database_account_header(request.http_request.headers)
                        and not request_params.healthy_tentative_location):
                    if retry_settings['connect'] > 0:
                        self.counter += 1
                        global_endpoint_manager.record_failure(request_params)
                        retry_active = self.increment(retry_settings, response=request, error=err)
                        if retry_active:
                            self.sleep(retry_settings, request.context.transport)
                            continue
                raise err
            except ServiceResponseError as err:
                retry_error = err
                # Only read operations can be safely retried with ServiceResponseError
                if (not _has_read_retryable_headers(request.http_request.headers) or
                        _has_database_account_header(request.http_request.headers) or
                        request_params.healthy_tentative_location):
                    raise err
                # This logic is based on the _retry.py file from azure-core
                if retry_settings['read'] > 0:
                    self.counter += 1
                    global_endpoint_manager.record_failure(request_params)
                    retry_active = self.increment(retry_settings, response=request, error=err)
                    if retry_active:
                        self.sleep(retry_settings, request.context.transport)
                        continue
                raise err
            except CosmosHttpResponseError as err:
                raise err
            except AzureError as err:
                retry_error = err
                if (_has_database_account_header(request.http_request.headers) or
                        request_params.healthy_tentative_location):
                    raise err
                if _has_read_retryable_headers(request.http_request.headers) and retry_settings['read'] > 0:
                    self.counter += 1
                    global_endpoint_manager.record_failure(request_params)
                    retry_active = self.increment(retry_settings, response=request, error=err)
                    if retry_active:
                        self.sleep(retry_settings, request.context.transport)
                        continue
                raise err
            finally:
                end_time = time.time()
                if absolute_timeout:
                    absolute_timeout -= (end_time - start_time)

        self.update_context(response.context, retry_settings)
        return response

class MockConnectionRetryPolicyAsync(AsyncRetryPolicy):

    def __init__(self, resource_type, error = None, **kwargs):
        self.resource_type = resource_type
        self.error = error
        self.counter = 0
        self.request_endpoints = []
        clean_kwargs = {k: v for k, v in kwargs.items() if v is not None}
        super(MockConnectionRetryPolicyAsync, self).__init__(**clean_kwargs)

    async def send(self, request):
        """Sends the PipelineRequest object to the next policy. Uses retry settings if necessary.
        Also enforces an absolute client-side timeout that spans multiple retry attempts.

        :param request: The PipelineRequest object
        :type request: ~azure.core.pipeline.PipelineRequest
        :return: Returns the PipelineResponse or raises error if maximum retries exceeded.
        :rtype: ~azure.core.pipeline.PipelineResponse
        :raises ~azure.core.exceptions.AzureError: Maximum retries exceeded.
        :raises ~azure.cosmos.exceptions.CosmosClientTimeoutError: Specified timeout exceeded.
        :raises ~azure.core.exceptions.ClientAuthenticationError: Authentication failed.
        """
        self.counter = 0
        absolute_timeout = request.context.options.pop('timeout', None)
        per_request_timeout = request.context.options.pop('connection_timeout', 0)
        request_params = request.context.options.pop('request_params', None)
        global_endpoint_manager = request.context.options.pop('global_endpoint_manager', None)
        retry_error = None
        retry_active = True
        response = None
        retry_settings = self.configure_retries(request.context.options)
        while retry_active:
            start_time = time.time()
            try:
                if request.http_request.headers.get(
                        http_constants.HttpHeaders.ThinClientProxyResourceType) == self.resource_type:
                    self.request_endpoints.append(request.http_request.url)
                    if self.error:
                        raise self.error
                _configure_timeout(request, absolute_timeout, per_request_timeout)
                response = await self.next.send(request)
                break
            except ClientAuthenticationError:  # pylint:disable=try-except-raise
                # the authentication policy failed such that the client's request can't
                # succeed--we'll never have a response to it, so propagate the exception
                raise
            except exceptions.CosmosClientTimeoutError as timeout_error:
                timeout_error.inner_exception = retry_error
                timeout_error.response = response
                timeout_error.history = retry_settings['history']
                raise
            except ServiceRequestError as err:
                retry_error = err
                # the request ran into a socket timeout or failed to establish a new connection
                # since request wasn't sent, raise exception immediately to be dealt with in client retry policies
                if (not _has_database_account_header(request.http_request.headers)
                        and not request_params.healthy_tentative_location):
                    if retry_settings['connect'] > 0:
                        self.counter += 1
                        await global_endpoint_manager.record_failure(request_params)
                        retry_active = self.increment(retry_settings, response=request, error=err)
                        if retry_active:
                            await self.sleep(retry_settings, request.context.transport)
                            continue
                raise err
            except ServiceResponseError as err:
                retry_error = err
                if (_has_database_account_header(request.http_request.headers) or
                        request_params.healthy_tentative_location):
                    raise err
                # Since this is ClientConnectionError, it is safe to be retried on both read and write requests
                try:
                    # pylint: disable=networking-import-outside-azure-core-transport
                    from aiohttp.client_exceptions import (
                        ClientConnectionError)
                    if (isinstance(err.inner_exception, ClientConnectionError)
                            or _has_read_retryable_headers(request.http_request.headers)):
                        # This logic is based on the _retry.py file from azure-core
                        if retry_settings['read'] > 0:
                            self.counter += 1
                            await global_endpoint_manager.record_failure(request_params)
                            retry_active = self.increment(retry_settings, response=request, error=err)
                            if retry_active:
                                await self.sleep(retry_settings, request.context.transport)
                                continue
                except ImportError:
                    raise err # pylint: disable=raise-missing-from
                raise err
            except CosmosHttpResponseError as err:
                raise err
            except AzureError as err:
                retry_error = err
                if (_has_database_account_header(request.http_request.headers) or
                        request_params.healthy_tentative_location):
                    raise err
                if _has_read_retryable_headers(request.http_request.headers) and retry_settings['read'] > 0:
                    self.counter += 1
                    retry_active = self.increment(retry_settings, response=request, error=err)
                    if retry_active:
                        await self.sleep(retry_settings, request.context.transport)
                        continue
                raise err
            finally:
                end_time = time.time()
                if absolute_timeout:
                    absolute_timeout -= (end_time - start_time)

        self.update_context(response.context, retry_settings)
        return response



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.containerregistry.v2025_04_01 import ContainerRegistryManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestContainerRegistryManagementRegistriesOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ContainerRegistryManagementClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_begin_import_image(self, resource_group):
        response = self.client.registries.begin_import_image(
            resource_group_name=resource_group.name,
            registry_name="str",
            parameters={
                "source": {
                    "sourceImage": "str",
                    "credentials": {"password": "str", "username": "str"},
                    "registryUri": "str",
                    "resourceId": "str",
                },
                "mode": "NoForce",
                "targetTags": ["str"],
                "untaggedTargetRepositories": ["str"],
            },
            api_version="2025-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_check_name_availability(self, resource_group):
        response = self.client.registries.check_name_availability(
            registry_name_check_request={"name": "str", "type": "Microsoft.ContainerRegistry/registries"},
            api_version="2025-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_list(self, resource_group):
        response = self.client.registries.list(
            api_version="2025-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_list_by_resource_group(self, resource_group):
        response = self.client.registries.list_by_resource_group(
            resource_group_name=resource_group.name,
            api_version="2025-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_get(self, resource_group):
        response = self.client.registries.get(
            resource_group_name=resource_group.name,
            registry_name="str",
            api_version="2025-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_begin_create(self, resource_group):
        response = self.client.registries.begin_create(
            resource_group_name=resource_group.name,
            registry_name="str",
            registry={
                "location": "str",
                "sku": {"name": "str", "tier": "str"},
                "adminUserEnabled": False,
                "anonymousPullEnabled": False,
                "creationDate": "2020-02-20 00:00:00",
                "dataEndpointEnabled": bool,
                "dataEndpointHostNames": ["str"],
                "encryption": {
                    "keyVaultProperties": {
                        "identity": "str",
                        "keyIdentifier": "str",
                        "keyRotationEnabled": bool,
                        "lastKeyRotationTimestamp": "2020-02-20 00:00:00",
                        "versionedKeyIdentifier": "str",
                    },
                    "status": "str",
                },
                "id": "str",
                "identity": {
                    "principalId": "str",
                    "tenantId": "str",
                    "type": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "loginServer": "str",
                "name": "str",
                "networkRuleBypassOptions": "str",
                "networkRuleSet": {"defaultAction": "Allow", "ipRules": [{"value": "str", "action": "Allow"}]},
                "policies": {
                    "azureADAuthenticationAsArmPolicy": {"status": "enabled"},
                    "exportPolicy": {"status": "enabled"},
                    "quarantinePolicy": {"status": "str"},
                    "retentionPolicy": {"days": 7, "lastUpdatedTime": "2020-02-20 00:00:00", "status": "str"},
                    "trustPolicy": {"status": "str", "type": "Notary"},
                },
                "privateEndpointConnections": [
                    {
                        "id": "str",
                        "name": "str",
                        "privateEndpoint": {"id": "str"},
                        "privateLinkServiceConnectionState": {
                            "actionsRequired": "str",
                            "description": "str",
                            "status": "str",
                        },
                        "provisioningState": "str",
                        "systemData": {
                            "createdAt": "2020-02-20 00:00:00",
                            "createdBy": "str",
                            "createdByType": "str",
                            "lastModifiedAt": "2020-02-20 00:00:00",
                            "lastModifiedBy": "str",
                            "lastModifiedByType": "str",
                        },
                        "type": "str",
                    }
                ],
                "provisioningState": "str",
                "publicNetworkAccess": "str",
                "status": {"displayStatus": "str", "message": "str", "timestamp": "2020-02-20 00:00:00"},
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
                "zoneRedundancy": "str",
            },
            api_version="2025-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_begin_delete(self, resource_group):
        response = self.client.registries.begin_delete(
            resource_group_name=resource_group.name,
            registry_name="str",
            api_version="2025-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_begin_update(self, resource_group):
        response = self.client.registries.begin_update(
            resource_group_name=resource_group.name,
            registry_name="str",
            registry_update_parameters={
                "adminUserEnabled": bool,
                "anonymousPullEnabled": bool,
                "dataEndpointEnabled": bool,
                "encryption": {
                    "keyVaultProperties": {
                        "identity": "str",
                        "keyIdentifier": "str",
                        "keyRotationEnabled": bool,
                        "lastKeyRotationTimestamp": "2020-02-20 00:00:00",
                        "versionedKeyIdentifier": "str",
                    },
                    "status": "str",
                },
                "identity": {
                    "principalId": "str",
                    "tenantId": "str",
                    "type": "str",
                    "userAssignedIdentities": {"str": {"clientId": "str", "principalId": "str"}},
                },
                "networkRuleBypassOptions": "str",
                "networkRuleSet": {"defaultAction": "Allow", "ipRules": [{"value": "str", "action": "Allow"}]},
                "policies": {
                    "azureADAuthenticationAsArmPolicy": {"status": "enabled"},
                    "exportPolicy": {"status": "enabled"},
                    "quarantinePolicy": {"status": "str"},
                    "retentionPolicy": {"days": 7, "lastUpdatedTime": "2020-02-20 00:00:00", "status": "str"},
                    "trustPolicy": {"status": "str", "type": "Notary"},
                },
                "publicNetworkAccess": "str",
                "sku": {"name": "str", "tier": "str"},
                "tags": {"str": "str"},
            },
            api_version="2025-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_list_usages(self, resource_group):
        response = self.client.registries.list_usages(
            resource_group_name=resource_group.name,
            registry_name="str",
            api_version="2025-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_list_private_link_resources(self, resource_group):
        response = self.client.registries.list_private_link_resources(
            resource_group_name=resource_group.name,
            registry_name="str",
            api_version="2025-04-01",
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_get_private_link_resource(self, resource_group):
        response = self.client.registries.get_private_link_resource(
            resource_group_name=resource_group.name,
            registry_name="str",
            group_name="str",
            api_version="2025-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_list_credentials(self, resource_group):
        response = self.client.registries.list_credentials(
            resource_group_name=resource_group.name,
            registry_name="str",
            api_version="2025-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_regenerate_credential(self, resource_group):
        response = self.client.registries.regenerate_credential(
            resource_group_name=resource_group.name,
            registry_name="str",
            regenerate_credential_parameters={"name": "str"},
            api_version="2025-04-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_registries_begin_generate_credentials(self, resource_group):
        response = self.client.registries.begin_generate_credentials(
            resource_group_name=resource_group.name,
            registry_name="str",
            generate_credentials_parameters={"expiry": "2020-02-20 00:00:00", "name": "str", "tokenId": "str"},
            api_version="2025-04-01",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=useless-super-delegation

import datetime
from typing import Any, List, Mapping, Optional, TYPE_CHECKING, Union, overload

from .. import _model_base
from .._model_base import rest_field

if TYPE_CHECKING:
    from .. import models as _models


class ErrorAdditionalInfo(_model_base.Model):
    """The resource management error additional info.

    :ivar type: The additional info type.
    :vartype type: str
    :ivar info: The additional info.
    :vartype info: any
    """

    type: Optional[str] = rest_field(visibility=["read"])
    """The additional info type."""
    info: Optional[Any] = rest_field(visibility=["read"])
    """The additional info."""


class ErrorDetail(_model_base.Model):
    """The error detail.

    :ivar code: The error code.
    :vartype code: str
    :ivar message: The error message.
    :vartype message: str
    :ivar target: The error target.
    :vartype target: str
    :ivar details: The error details.
    :vartype details: list[~azure.mgmt.edgezones.models.ErrorDetail]
    :ivar additional_info: The error additional info.
    :vartype additional_info: list[~azure.mgmt.edgezones.models.ErrorAdditionalInfo]
    """

    code: Optional[str] = rest_field(visibility=["read"])
    """The error code."""
    message: Optional[str] = rest_field(visibility=["read"])
    """The error message."""
    target: Optional[str] = rest_field(visibility=["read"])
    """The error target."""
    details: Optional[List["_models.ErrorDetail"]] = rest_field(visibility=["read"])
    """The error details."""
    additional_info: Optional[List["_models.ErrorAdditionalInfo"]] = rest_field(
        name="additionalInfo", visibility=["read"]
    )
    """The error additional info."""


class ErrorResponse(_model_base.Model):
    """Common error response for all Azure Resource Manager APIs to return error details for failed
    operations.

    :ivar error: The error object.
    :vartype error: ~azure.mgmt.edgezones.models.ErrorDetail
    """

    error: Optional["_models.ErrorDetail"] = rest_field(visibility=["read", "create", "update", "delete", "query"])
    """The error object."""

    @overload
    def __init__(
        self,
        *,
        error: Optional["_models.ErrorDetail"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class Resource(_model_base.Model):
    """Common fields that are returned in the response for all Azure Resource Manager resources.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.mgmt.edgezones.models.SystemData
    """

    id: Optional[str] = rest_field(visibility=["read"])
    """Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}."""
    name: Optional[str] = rest_field(visibility=["read"])
    """The name of the resource."""
    type: Optional[str] = rest_field(visibility=["read"])
    """The type of the resource. E.g. \"Microsoft.Compute/virtualMachines\" or
     \"Microsoft.Storage/storageAccounts\"."""
    system_data: Optional["_models.SystemData"] = rest_field(name="systemData", visibility=["read"])
    """Azure Resource Manager metadata containing createdBy and modifiedBy information."""


class ProxyResource(Resource):
    """The resource model definition for a Azure Resource Manager proxy resource. It will not have
    tags and a location.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.mgmt.edgezones.models.SystemData
    """


class ExtendedZone(ProxyResource):
    """Resource that represents an Azure Extended Zone available to a subscription for registering and
    unregistering.

    :ivar id: Fully qualified resource ID for the resource. Ex -
     /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}.
    :vartype id: str
    :ivar name: The name of the resource.
    :vartype name: str
    :ivar type: The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or
     "Microsoft.Storage/storageAccounts".
    :vartype type: str
    :ivar system_data: Azure Resource Manager metadata containing createdBy and modifiedBy
     information.
    :vartype system_data: ~azure.mgmt.edgezones.models.SystemData
    :ivar properties: The resource-specific properties for this resource.
    :vartype properties: ~azure.mgmt.edgezones.models.ExtendedZoneProperties
    """

    properties: Optional["_models.ExtendedZoneProperties"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """The resource-specific properties for this resource."""

    @overload
    def __init__(
        self,
        *,
        properties: Optional["_models.ExtendedZoneProperties"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class ExtendedZoneProperties(_model_base.Model):
    """The properties of an Extended Zone resource.

    :ivar provisioning_state: Status of the last operation performed by the subscription on the
     Edge Zone resource. Known values are: "Succeeded", "Failed", "Canceled", "Provisioning",
     "Updating", "Deleting", and "Accepted".
    :vartype provisioning_state: str or ~azure.mgmt.edgezones.models.ProvisioningState
    :ivar registration_state: Indicates the Azure Extended Zone registration’s approval status.
     Known values are: "NotRegistered", "PendingRegister", "Registered", and "PendingUnregister".
    :vartype registration_state: str or ~azure.mgmt.edgezones.models.RegistrationState
    :ivar display_name: Display name of the Azure Extended Zone. Required.
    :vartype display_name: str
    :ivar regional_display_name: Regional display name of the Azure Extended Zone. Required.
    :vartype regional_display_name: str
    :ivar region_type: Type of region for the Azure Extended Zone. Required.
    :vartype region_type: str
    :ivar region_category: Category of region for the Azure Extended Zone. Required.
    :vartype region_category: str
    :ivar geography: Geography of the Azure Extended Zone. Required.
    :vartype geography: str
    :ivar geography_group: The Geography Group of the Azure Extended Zone. Required.
    :vartype geography_group: str
    :ivar longitude: The Longitude of the Azure Extended Zone. Required.
    :vartype longitude: str
    :ivar latitude: The Latitude of the Azure Extended Zone. Required.
    :vartype latitude: str
    :ivar home_location: The Home Location of the Azure Extended Zone. Required.
    :vartype home_location: str
    """

    provisioning_state: Optional[Union[str, "_models.ProvisioningState"]] = rest_field(
        name="provisioningState", visibility=["read"]
    )
    """Status of the last operation performed by the subscription on the Edge Zone resource. Known
     values are: \"Succeeded\", \"Failed\", \"Canceled\", \"Provisioning\", \"Updating\",
     \"Deleting\", and \"Accepted\"."""
    registration_state: Optional[Union[str, "_models.RegistrationState"]] = rest_field(
        name="registrationState", visibility=["read"]
    )
    """Indicates the Azure Extended Zone registration’s approval status. Known values are:
     \"NotRegistered\", \"PendingRegister\", \"Registered\", and \"PendingUnregister\"."""
    display_name: str = rest_field(name="displayName", visibility=["read"])
    """Display name of the Azure Extended Zone. Required."""
    regional_display_name: str = rest_field(name="regionalDisplayName", visibility=["read"])
    """Regional display name of the Azure Extended Zone. Required."""
    region_type: str = rest_field(name="regionType", visibility=["read"])
    """Type of region for the Azure Extended Zone. Required."""
    region_category: str = rest_field(name="regionCategory", visibility=["read"])
    """Category of region for the Azure Extended Zone. Required."""
    geography: str = rest_field(visibility=["read"])
    """Geography of the Azure Extended Zone. Required."""
    geography_group: str = rest_field(name="geographyGroup", visibility=["read"])
    """The Geography Group of the Azure Extended Zone. Required."""
    longitude: str = rest_field(visibility=["read"])
    """The Longitude of the Azure Extended Zone. Required."""
    latitude: str = rest_field(visibility=["read"])
    """The Latitude of the Azure Extended Zone. Required."""
    home_location: str = rest_field(name="homeLocation", visibility=["read"])
    """The Home Location of the Azure Extended Zone. Required."""


class Operation(_model_base.Model):
    """Details of a REST API operation, returned from the Resource Provider Operations API.

    :ivar name: The name of the operation, as per Resource-Based Access Control (RBAC). Examples:
     "Microsoft.Compute/virtualMachines/write", "Microsoft.Compute/virtualMachines/capture/action".
    :vartype name: str
    :ivar is_data_action: Whether the operation applies to data-plane. This is "true" for
     data-plane operations and "false" for Azure Resource Manager/control-plane operations.
    :vartype is_data_action: bool
    :ivar display: Localized display information for this particular operation.
    :vartype display: ~azure.mgmt.edgezones.models.OperationDisplay
    :ivar origin: The intended executor of the operation; as in Resource Based Access Control
     (RBAC) and audit logs UX. Default value is "user,system". Known values are: "user", "system",
     and "user,system".
    :vartype origin: str or ~azure.mgmt.edgezones.models.Origin
    :ivar action_type: Extensible enum. Indicates the action type. "Internal" refers to actions
     that are for internal only APIs. "Internal"
    :vartype action_type: str or ~azure.mgmt.edgezones.models.ActionType
    """

    name: Optional[str] = rest_field(visibility=["read"])
    """The name of the operation, as per Resource-Based Access Control (RBAC). Examples:
     \"Microsoft.Compute/virtualMachines/write\",
     \"Microsoft.Compute/virtualMachines/capture/action\"."""
    is_data_action: Optional[bool] = rest_field(name="isDataAction", visibility=["read"])
    """Whether the operation applies to data-plane. This is \"true\" for data-plane operations and
     \"false\" for Azure Resource Manager/control-plane operations."""
    display: Optional["_models.OperationDisplay"] = rest_field(
        visibility=["read", "create", "update", "delete", "query"]
    )
    """Localized display information for this particular operation."""
    origin: Optional[Union[str, "_models.Origin"]] = rest_field(visibility=["read"])
    """The intended executor of the operation; as in Resource Based Access Control (RBAC) and audit
     logs UX. Default value is \"user,system\". Known values are: \"user\", \"system\", and
     \"user,system\"."""
    action_type: Optional[Union[str, "_models.ActionType"]] = rest_field(name="actionType", visibility=["read"])
    """Extensible enum. Indicates the action type. \"Internal\" refers to actions that are for
     internal only APIs. \"Internal\""""

    @overload
    def __init__(
        self,
        *,
        display: Optional["_models.OperationDisplay"] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)


class OperationDisplay(_model_base.Model):
    """Localized display information for and operation.

    :ivar provider: The localized friendly form of the resource provider name, e.g. "Microsoft
     Monitoring Insights" or "Microsoft Compute".
    :vartype provider: str
    :ivar resource: The localized friendly name of the resource type related to this operation.
     E.g. "Virtual Machines" or "Job Schedule Collections".
    :vartype resource: str
    :ivar operation: The concise, localized friendly name for the operation; suitable for
     dropdowns. E.g. "Create or Update Virtual Machine", "Restart Virtual Machine".
    :vartype operation: str
    :ivar description: The short, localized friendly description of the operation; suitable for
     tool tips and detailed views.
    :vartype description: str
    """

    provider: Optional[str] = rest_field(visibility=["read"])
    """The localized friendly form of the resource provider name, e.g. \"Microsoft Monitoring
     Insights\" or \"Microsoft Compute\"."""
    resource: Optional[str] = rest_field(visibility=["read"])
    """The localized friendly name of the resource type related to this operation. E.g. \"Virtual
     Machines\" or \"Job Schedule Collections\"."""
    operation: Optional[str] = rest_field(visibility=["read"])
    """The concise, localized friendly name for the operation; suitable for dropdowns. E.g. \"Create
     or Update Virtual Machine\", \"Restart Virtual Machine\"."""
    description: Optional[str] = rest_field(visibility=["read"])
    """The short, localized friendly description of the operation; suitable for tool tips and detailed
     views."""


class SystemData(_model_base.Model):
    """Metadata pertaining to creation and last modification of the resource.

    :ivar created_by: The identity that created the resource.
    :vartype created_by: str
    :ivar created_by_type: The type of identity that created the resource. Known values are:
     "User", "Application", "ManagedIdentity", and "Key".
    :vartype created_by_type: str or ~azure.mgmt.edgezones.models.CreatedByType
    :ivar created_at: The timestamp of resource creation (UTC).
    :vartype created_at: ~datetime.datetime
    :ivar last_modified_by: The identity that last modified the resource.
    :vartype last_modified_by: str
    :ivar last_modified_by_type: The type of identity that last modified the resource. Known values
     are: "User", "Application", "ManagedIdentity", and "Key".
    :vartype last_modified_by_type: str or ~azure.mgmt.edgezones.models.CreatedByType
    :ivar last_modified_at: The timestamp of resource last modification (UTC).
    :vartype last_modified_at: ~datetime.datetime
    """

    created_by: Optional[str] = rest_field(name="createdBy", visibility=["read", "create", "update", "delete", "query"])
    """The identity that created the resource."""
    created_by_type: Optional[Union[str, "_models.CreatedByType"]] = rest_field(
        name="createdByType", visibility=["read", "create", "update", "delete", "query"]
    )
    """The type of identity that created the resource. Known values are: \"User\", \"Application\",
     \"ManagedIdentity\", and \"Key\"."""
    created_at: Optional[datetime.datetime] = rest_field(
        name="createdAt", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The timestamp of resource creation (UTC)."""
    last_modified_by: Optional[str] = rest_field(
        name="lastModifiedBy", visibility=["read", "create", "update", "delete", "query"]
    )
    """The identity that last modified the resource."""
    last_modified_by_type: Optional[Union[str, "_models.CreatedByType"]] = rest_field(
        name="lastModifiedByType", visibility=["read", "create", "update", "delete", "query"]
    )
    """The type of identity that last modified the resource. Known values are: \"User\",
     \"Application\", \"ManagedIdentity\", and \"Key\"."""
    last_modified_at: Optional[datetime.datetime] = rest_field(
        name="lastModifiedAt", visibility=["read", "create", "update", "delete", "query"], format="rfc3339"
    )
    """The timestamp of resource last modification (UTC)."""

    @overload
    def __init__(
        self,
        *,
        created_by: Optional[str] = None,
        created_by_type: Optional[Union[str, "_models.CreatedByType"]] = None,
        created_at: Optional[datetime.datetime] = None,
        last_modified_by: Optional[str] = None,
        last_modified_by_type: Optional[Union[str, "_models.CreatedByType"]] = None,
        last_modified_at: Optional[datetime.datetime] = None,
    ) -> None: ...

    @overload
    def __init__(self, mapping: Mapping[str, Any]) -> None:
        """
        :param mapping: raw JSON to initialize the model.
        :type mapping: Mapping[str, Any]
        """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
# mypy: disable-error-code="import-untyped,return-value"
# pylint: disable=line-too-long,R,wrong-import-order,global-variable-not-assigned)
import json
import os
import re
import sys
from typing import Any, Dict
from pathlib import Path


_yaml_regex = re.compile(
    r"^\s*" + r"(?:---|\+\+\+)" + r"(.*?)" + r"(?:---|\+\+\+)" + r"\s*(.+)$",
    re.S | re.M,
)


def load_text(file_path, encoding="utf-8"):
    with open(file_path, "r", encoding=encoding) as file:
        return file.read()


def load_json(file_path, encoding="utf-8"):
    return json.loads(load_text(file_path, encoding=encoding))


def load_global_config(prompty_path: Path = Path.cwd(), configuration: str = "default") -> Dict[str, Any]:
    prompty_config_path = prompty_path.joinpath("prompty.json")
    if os.path.exists(prompty_config_path):
        c = load_json(prompty_config_path)
        if configuration in c:
            return c[configuration]
        else:
            raise ValueError(f'Item "{configuration}" not found in "{prompty_config_path}"')
    else:
        return {}


def load_prompty(file_path, encoding="utf-8") -> Dict[str, Any]:
    contents = load_text(file_path, encoding=encoding)
    return parse(contents)


def parse(contents):
    try:
        import yaml  # type: ignore
    except ImportError as exc:
        raise ImportError("Please install pyyaml to use this function. Run `pip install pyyaml`.") from exc

    global _yaml_regex

    fmatter = ""
    body = ""
    result = _yaml_regex.search(contents)

    if result:
        fmatter = result.group(1)
        body = result.group(2)
    return {
        "attributes": yaml.load(fmatter, Loader=yaml.SafeLoader),
        "body": body,
        "frontmatter": fmatter,
    }


def remove_leading_empty_space(multiline_str: str) -> str:
    """
    Processes a multiline string by:
    1. Removing empty lines
    2. Finding the minimum leading spaces
    3. Indenting all lines to the minimum level

    :param multiline_str: The input multiline string.
    :type multiline_str: str
    :return: The processed multiline string.
    :rtype: str
    """
    lines = multiline_str.splitlines()
    start_index = 0
    while start_index < len(lines) and lines[start_index].strip() == "":
        start_index += 1

    # Find the minimum number of leading spaces
    min_spaces = sys.maxsize
    for line in lines[start_index:]:
        if len(line.strip()) == 0:
            continue
        spaces = len(line) - len(line.lstrip())
        spaces += line.lstrip().count("\t") * 2  # Count tabs as 2 spaces
        min_spaces = min(min_spaces, spaces)

    # Remove leading spaces and indent to the minimum level
    processed_lines = []
    for line in lines[start_index:]:
        processed_lines.append(line[min_spaces:])

    return "\n".join(processed_lines)



# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------
import json
import random
import base64
import hashlib
from dataclasses import dataclass
from typing import Dict, List, Optional, Mapping, Any
from azure.appconfiguration import (  # type:ignore # pylint:disable=no-name-in-module
    FeatureFlagConfigurationSetting,
)
from ._constants import (
    PERCENTAGE_FILTER_NAMES,
    TIME_WINDOW_FILTER_NAMES,
    TARGETING_FILTER_NAMES,
    CUSTOM_FILTER_KEY,
    PERCENTAGE_FILTER_KEY,
    TIME_WINDOW_FILTER_KEY,
    TARGETING_FILTER_KEY,
    TELEMETRY_KEY,
    METADATA_KEY,
    ETAG_KEY,
    FEATURE_FLAG_REFERENCE_KEY,
    ALLOCATION_ID_KEY,
)

FALLBACK_CLIENT_REFRESH_EXPIRED_INTERVAL = 3600  # 1 hour in seconds
MINIMAL_CLIENT_REFRESH_INTERVAL = 30  # 30 seconds

JSON = Mapping[str, Any]


@dataclass
class _ConfigurationClientWrapperBase:
    endpoint: str

    @staticmethod
    def _generate_allocation_id(feature_flag_value: Dict[str, JSON]) -> Optional[str]:
        """
        Generates an allocation ID for the specified feature.
        seed=123abc\ndefault_when_enabled=Control\npercentiles=0,Control,20;20,Test,100\nvariants=Control,standard;Test,special # pylint:disable=line-too-long

        :param  Dict[str, JSON] feature_flag_value: The feature to generate an allocation ID for.
        :rtype: str
        :return: The allocation ID.
        """

        allocation_id = ""
        allocated_variants = []

        allocation: Optional[JSON] = feature_flag_value.get("allocation")

        if not allocation:
            return None

        # Seed
        allocation_id = f"seed={allocation.get('seed', '')}"

        # DefaultWhenEnabled
        if "default_when_enabled" in allocation:
            allocated_variants.append(allocation.get("default_when_enabled"))

        allocation_id += f"\ndefault_when_enabled={allocation.get('default_when_enabled', '')}"

        # Percentile
        allocation_id += "\npercentiles="

        percentile = allocation.get("percentile")

        if percentile:
            percentile_allocations = sorted(
                (x for x in percentile if x.get("from") != x.get("to")),
                key=lambda x: x.get("from"),
            )

            for percentile_allocation in percentile_allocations:
                if "variant" in percentile_allocation:
                    allocated_variants.append(percentile_allocation.get("variant"))

            allocation_id += ";".join(
                f"{pa.get('from')}," f"{base64.b64encode(pa.get('variant').encode()).decode()}," f"{pa.get('to')}"
                for pa in percentile_allocations
            )

        if not allocated_variants and not allocation.get("seed"):
            return None

        # Variants
        allocation_id += "\nvariants="

        variants_value = feature_flag_value.get("variants")
        if variants_value and (isinstance(variants_value, list) or all(isinstance(v, dict) for v in variants_value)):
            if (
                allocated_variants
                and isinstance(variants_value, list)
                and all(isinstance(v, dict) for v in variants_value)
            ):
                sorted_variants: List[Dict[str, Any]] = sorted(
                    (v for v in variants_value if v.get("name") in allocated_variants),
                    key=lambda v: v.get("name"),
                )

                for v in sorted_variants:
                    allocation_id += f"{base64.b64encode(v.get('name', '').encode()).decode()},"
                    if "configuration_value" in v:
                        allocation_id += (
                            f"{json.dumps(v.get('configuration_value', ''), separators=(',', ':'), sort_keys=True)}"
                        )
                    allocation_id += ";"
                if sorted_variants:
                    allocation_id = allocation_id[:-1]

        # Create a sha256 hash of the allocation_id
        hash_object = hashlib.sha256(allocation_id.encode())
        hash_digest = hash_object.digest()

        # Encode the first 15 bytes in base64 url
        return base64.urlsafe_b64encode(hash_digest[:15]).decode()

    def _feature_flag_telemetry(
        self, endpoint: str, feature_flag: FeatureFlagConfigurationSetting, feature_flag_value: Dict
    ):
        if TELEMETRY_KEY in feature_flag_value:
            if METADATA_KEY not in feature_flag_value[TELEMETRY_KEY]:
                feature_flag_value[TELEMETRY_KEY][METADATA_KEY] = {}
            feature_flag_value[TELEMETRY_KEY][METADATA_KEY][ETAG_KEY] = feature_flag.etag

            if not endpoint.endswith("/"):
                endpoint += "/"
            feature_flag_reference = f"{endpoint}kv/{feature_flag.key}"
            if feature_flag.label and not feature_flag.label.isspace():
                feature_flag_reference += f"?label={feature_flag.label}"
            if feature_flag_value[TELEMETRY_KEY].get("enabled"):
                feature_flag_value[TELEMETRY_KEY][METADATA_KEY][FEATURE_FLAG_REFERENCE_KEY] = feature_flag_reference
                allocation_id = self._generate_allocation_id(feature_flag_value)
                if allocation_id:
                    feature_flag_value[TELEMETRY_KEY][METADATA_KEY][ALLOCATION_ID_KEY] = allocation_id

    def _feature_flag_appconfig_telemetry(
        self, feature_flag: FeatureFlagConfigurationSetting, filters_used: Dict[str, bool]
    ):
        if feature_flag.filters:
            for filter in feature_flag.filters:
                if filter.get("name") in PERCENTAGE_FILTER_NAMES:
                    filters_used[PERCENTAGE_FILTER_KEY] = True
                elif filter.get("name") in TIME_WINDOW_FILTER_NAMES:
                    filters_used[TIME_WINDOW_FILTER_KEY] = True
                elif filter.get("name") in TARGETING_FILTER_NAMES:
                    filters_used[TARGETING_FILTER_KEY] = True
                else:
                    filters_used[CUSTOM_FILTER_KEY] = True


class ConfigurationClientManagerBase:  # pylint:disable=too-many-instance-attributes
    def __init__(
        self,
        endpoint: str,
        user_agent: str,
        retry_total,
        retry_backoff_max,
        replica_discovery_enabled,
        min_backoff_sec,
        max_backoff_sec,
        _load_balancing_enabled: bool,
        **kwargs,
    ):
        self._last_active_client_name = ""
        self._original_endpoint = endpoint
        self._user_agent = user_agent
        self._retry_total = retry_total
        self._retry_backoff_max = retry_backoff_max
        self._replica_discovery_enabled = replica_discovery_enabled
        self._next_update_time: Optional[float] = None
        self._args = dict(kwargs)
        self._min_backoff_sec = min_backoff_sec
        self._max_backoff_sec = max_backoff_sec
        self._load_balancing_enabled = _load_balancing_enabled

    def _calculate_backoff(self, attempts: int) -> float:
        max_attempts = 63
        ms_per_second = 1000  # 1 Second in milliseconds

        min_backoff_milliseconds = self._min_backoff_sec * ms_per_second
        max_backoff_milliseconds = self._max_backoff_sec * ms_per_second

        if self._max_backoff_sec <= self._min_backoff_sec:
            return min_backoff_milliseconds

        calculated_milliseconds = max(1, min_backoff_milliseconds) * (1 << min(attempts, max_attempts))

        if calculated_milliseconds > max_backoff_milliseconds or calculated_milliseconds <= 0:
            calculated_milliseconds = max_backoff_milliseconds

        return min_backoff_milliseconds + (
            random.uniform(0.0, 1.0) * (calculated_milliseconds - min_backoff_milliseconds)  # nosec
        )



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import msrest.serialization


class SmsSendResult(msrest.serialization.Model):
    """Response for a single recipient.

    All required parameters must be populated in order to send to Azure.

    :param to: Required. The recipient's phone number in E.164 format.
    :type to: str
    :param message_id: The identifier of the outgoing Sms message. Only present if message
     processed.
    :type message_id: str
    :param http_status_code: Required. HTTP Status code.
    :type http_status_code: int
    :param successful: Required. Indicates if the message is processed successfully or not.
    :type successful: bool
    :param error_message: Optional error message in case of 4xx/5xx/repeatable errors.
    :type error_message: str
    """

    _validation = {
        "to": {"required": True},
        "http_status_code": {"required": True},
        "successful": {"required": True},
    }

    _attribute_map = {
        "to": {"key": "to", "type": "str"},
        "message_id": {"key": "messageId", "type": "str"},
        "http_status_code": {"key": "httpStatusCode", "type": "int"},
        "successful": {"key": "successful", "type": "bool"},
        "error_message": {"key": "errorMessage", "type": "str"},
    }

    def __init__(self, **kwargs):
        super(SmsSendResult, self).__init__(**kwargs)
        self.to = kwargs["to"]
        self.message_id = kwargs.get("message_id", None)
        self.http_status_code = kwargs["http_status_code"]
        self.successful = kwargs["successful"]
        self.error_message = kwargs.get("error_message", None)



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_analyze_general_documents_async.py

DESCRIPTION:
    This sample demonstrates how to extract general document information from a document
    given through a file.

USAGE:
    python sample_analyze_general_documents_async.py

    Set the environment variables with your own values before running the sample:
    1) DOCUMENTINTELLIGENCE_ENDPOINT - the endpoint to your Document Intelligence resource.
    2) DOCUMENTINTELLIGENCE_API_KEY - your Document Intelligence API key.
"""

import asyncio
import os


def get_words(words, line):
    result = []
    for word in words:
        if _in_span(word, line.spans):
            result.append(word)
    return result


def _in_span(word, spans):
    for span in spans:
        if word.span.offset >= span.offset and (word.span.offset + word.span.length) <= (span.offset + span.length):
            return True
    return False


def format_bounding_region(bounding_regions):
    if not bounding_regions:
        return "N/A"
    return ", ".join(f"Page #{region.page_number}: {format_polygon(region.polygon)}" for region in bounding_regions)


def format_polygon(polygon):
    if not polygon:
        return "N/A"
    return ", ".join([f"[{polygon[i]}, {polygon[i + 1]}]" for i in range(0, len(polygon), 2)])


async def analyze_general_documents():
    path_to_sample_documents = os.path.abspath(
        os.path.join(
            os.path.abspath(__file__),
            "..",
            "..",
            "./sample_forms/forms/form_selection_mark.png",
        )
    )

    from azure.core.credentials import AzureKeyCredential
    from azure.ai.documentintelligence.aio import DocumentIntelligenceClient
    from azure.ai.documentintelligence.models import DocumentAnalysisFeature, AnalyzeResult

    endpoint = os.environ["DOCUMENTINTELLIGENCE_ENDPOINT"]
    key = os.environ["DOCUMENTINTELLIGENCE_API_KEY"]

    document_intelligence_client = DocumentIntelligenceClient(endpoint=endpoint, credential=AzureKeyCredential(key))
    async with document_intelligence_client:
        with open(path_to_sample_documents, "rb") as f:
            poller = await document_intelligence_client.begin_analyze_document(
                "prebuilt-layout",
                body=f,
                features=[DocumentAnalysisFeature.KEY_VALUE_PAIRS],
            )
        result: AnalyzeResult = await poller.result()

    if result.styles:
        for style in result.styles:
            if style.is_handwritten:
                print("Document contains handwritten content: ")
                print(",".join([result.content[span.offset : span.offset + span.length] for span in style.spans]))

    print("----Key-value pairs found in document----")
    if result.key_value_pairs:
        for kv_pair in result.key_value_pairs:
            if kv_pair.key:
                print(
                    f"Key '{kv_pair.key.content}' found within "
                    f"'{format_bounding_region(kv_pair.key.bounding_regions)}' bounding regions"
                )
            if kv_pair.value:
                print(
                    f"Value '{kv_pair.value.content}' found within "
                    f"'{format_bounding_region(kv_pair.value.bounding_regions)}' bounding regions\n"
                )

    for page in result.pages:
        print(f"----Analyzing document from page #{page.page_number}----")
        print(f"Page has width: {page.width} and height: {page.height}, measured with unit: {page.unit}")

        if page.lines:
            for line_idx, line in enumerate(page.lines):
                words = get_words(page.words, line)
                print(
                    f"...Line #{line_idx} has {len(words)} words and text '{line.content}' within "
                    f"bounding polygon '{format_polygon(line.polygon)}'"
                )

        if page.words:
            for word in page.words:
                print(f"......Word '{word.content}' has a confidence of {word.confidence}")

        if page.selection_marks:
            for selection_mark in page.selection_marks:
                print(
                    f"Selection mark is '{selection_mark.state}' within bounding polygon "
                    f"'{format_polygon(selection_mark.polygon)}' and has a confidence of "
                    f"{selection_mark.confidence}"
                )

    if result.tables:
        for table_idx, table in enumerate(result.tables):
            print(f"Table # {table_idx} has {table.row_count} rows and {table.column_count} columns")
            if table.bounding_regions:
                for region in table.bounding_regions:
                    print(
                        f"Table # {table_idx} location on page: {region.page_number} is {format_polygon(region.polygon)}"
                    )
            for cell in table.cells:
                print(f"...Cell[{cell.row_index}][{cell.column_index}] has text '{cell.content}'")
                if cell.bounding_regions:
                    for region in cell.bounding_regions:
                        print(
                            f"...content on page {region.page_number} is within bounding polygon '{format_polygon(region.polygon)}'\n"
                        )
    print("----------------------------------------")


async def main():
    await analyze_general_documents()


if __name__ == "__main__":
    from azure.core.exceptions import HttpResponseError
    from dotenv import find_dotenv, load_dotenv

    try:
        load_dotenv(find_dotenv())
        asyncio.run(main())
    except HttpResponseError as error:
        # Examples of how to check an HttpResponseError
        # Check by error code:
        if error.error is not None:
            if error.error.code == "InvalidImage":
                print(f"Received an invalid image error: {error.error}")
            if error.error.code == "InvalidRequest":
                print(f"Received an invalid request error: {error.error}")
            # Raise the error again after printing it
            raise
        # If the inner error is None and then it is possible to check the message to get more information:
        if "Invalid request".casefold() in error.message.casefold():
            print(f"Uh-oh! Seems there was an invalid request: {error}")
        # Raise the error again
        raise
